"""
// @author: ComPleHN
// @file: gauge.vue
// @time: 2025/5/27 15:47
// @description: 本文件是对于echarts的 仪表盘 图表测试
"""
# ---------------------------------------------- Gauge - Gauge ----------------------------------------------
import pyecharts.options as opts
from pyecharts.charts import Gauge

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=gauge

目前无法实现的功能:

1、暂无
"""

c1 = (
    Gauge()
    .add(series_name="业务指标", data_pair=[["完成率", 55.5]])
    .set_global_opts(
        legend_opts=opts.LegendOpts(is_show=False),
        tooltip_opts=opts.TooltipOpts(is_show=True, formatter="{a} <br/>{b} : {c}%"),
    )
)

# ---------------------------------------------- Gauge - Gauge_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Gauge

c2 = (
    Gauge()
    .add("", [("完成率", 66.6)])
    .set_global_opts(title_opts=opts.TitleOpts(title="Gauge-基本示例"))
)

# ---------------------------------------------- Gauge - Gauge_color ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Gauge

c3 = (
    Gauge()
    .add(
        "业务指标",
        [("完成率", 55.5)],
        axisline_opts=opts.AxisLineOpts(
            linestyle_opts=opts.LineStyleOpts(
                color=[(0.3, "#67e0e3"), (0.7, "#37a2da"), (1, "#fd666d")], width=30
            )
        ),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Gauge-不同颜色"),
        legend_opts=opts.LegendOpts(is_show=False),
    )
)

# ---------------------------------------------- Gauge - Gauge_label_title_setting ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Gauge

c4 = (
    Gauge()
    .add(
        "",
        [("完成率", 66.6)],
        title_label_opts=opts.LabelOpts(
            font_size=40, color="blue", font_family="Microsoft YaHei"
        ),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Gauge-改变轮盘内的字体"))
)

# ---------------------------------------------- Gauge - Gauge_change_color ----------------------------------------------
import pyecharts.options as opts
from pyecharts.charts import Gauge

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://gallery.echartsjs.com/editor.html?c=xH1vxib94f

目前无法实现的功能:

1、暂无
"""

c5 = (
    Gauge()
    .add(series_name="业务指标", data_pair=[["完成率", 55.5]])
    .set_global_opts(
        legend_opts=opts.LegendOpts(is_show=False),
        tooltip_opts=opts.TooltipOpts(is_show=True, formatter="{a} <br/>{b} : {c}%"),
    )
    .set_series_opts(
        axisline_opts=opts.AxisLineOpts(
            linestyle_opts=opts.LineStyleOpts(
                color=[[0.3, "#67e0e3"], [0.7, "#37a2da"], [1, "#fd666d"]], width=30
            )
        )
    )
)

# ---------------------------------------------- Gauge - Gauge_splitnum_label ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Gauge

c6 = (
    Gauge()
    .add(
        "业务指标",
        [("完成率", 55.5)],
        split_number=5,
        axisline_opts=opts.AxisLineOpts(
            linestyle_opts=opts.LineStyleOpts(
                color=[(0.3, "#67e0e3"), (0.7, "#37a2da"), (1, "#fd666d")], width=30
            )
        ),
        detail_label_opts=opts.LabelOpts(formatter="{value}"),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Gauge-分割段数-Label"),
        legend_opts=opts.LegendOpts(is_show=False),
    )
)

# ---------------------------------------------- Gauge - Gauge_change_radius ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Gauge

c7 = (
    Gauge()
    .add("", [("完成率", 66.6)], radius="50%")
    .set_global_opts(title_opts=opts.TitleOpts(title="Gauge-修改 Radius 为 50%"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="gauge",
    public=True,
)

swanlab.log(
    {
        "Gauge - Gauge": c1,
        "Gauge - Gauge_base": c2,
        "Gauge - Gauge_color": c3,
        "Gauge - Gauge_label_title_setting": c4,
        "Gauge - Gauge_change_color": c5,
        "Gauge - Gauge_splitnum_label": c6,
        "Gauge - Gauge_change_radius": c7

    }
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2025/5/1 17:30
@File: experiment.py
@IDE: pycharm
@Description:
    实验相关的开放API
"""
from swanlab.api.base import ApiBase, ApiHTTP
from swanlab.api.types import ApiResponse, Experiment, Pagination


class ExperimentAPI(ApiBase):
    def __init__(self, http: ApiHTTP):
        super().__init__(http)

    @classmethod
    def parse(cls, body: dict) -> Experiment:
        return Experiment.model_validate({
            "cuid": body.get("cuid") or "",
            "name": body.get("name") or "",
            "description": body.get("description") or "",
            "state": body.get("state") or "",
            "show": bool(body.get("show")),
            "createdAt": body.get("createdAt") or "",
            "finishedAt": body.get("finishedAt") or "",
            "user": {
                "username": (body.get("user") or {}).get("username") or "",
                "name": (body.get("user") or {}).get("name") or "",
            },
            "profile": body.get("profile") or {}
        })

    def get_experiment(
            self,
            username: str,
            projname: str,
            exp_id: str
    ) -> ApiResponse[Experiment]:
        """
        获取实验信息

        Args:
            username (str): 工作空间名
            projname (str): 项目名
            exp_id (str): 实验CUID
        """
        resp: ApiResponse = self.http.service.get_exp_info(username=username, project=projname, exp_id=exp_id)
        if resp.errmsg:
            return resp
        resp.data = ExperimentAPI.parse(resp.data)
        return resp

    def delete_experiment(
            self,
            username: str,
            projname: str,
            exp_id: str
    ):
        """
        删除实验

        Args:
            username (str): 工作空间名
            projname (str): 项目名
            exp_id (str): 实验CUID
        """
        return self.http.delete(f"/project/{username}/{projname}/runs/{exp_id}", params={})

    def list_experiments(
            self,
            username: str,
            projname: str,
            page: int = 1,
            size: int = 10
    ) -> ApiResponse[Pagination[Experiment]]:
        """
        分页获取项目下的实验列表
        该接口返回的实验profile只包含config(用户自定义的配置)

        Args:
            username (str): 工作空间名
            projname (str): 项目名
            page (int): 页码, 默认为1
            size (int): 每页大小, 默认为10
        """
        resp = self.http.get(f"/project/{username}/{projname}/runs", params={"page": page, "size": size})

        if resp.errmsg:
            return resp
        exps = resp.data
        resp.data = Pagination[Experiment].model_validate({
            "total": exps.get("total", 0),
            "list": [ExperimentAPI.parse(e) for e in exps.get("list", [])]
        })
        return resp

    def get_summary(
            self,
            exp_id: str,
            pro_id: str,
            root_exp_id: str,
            root_pro_id: str
    ) -> ApiResponse[dict]:
        """
        获取实验的summary信息
        从House获取, 需要考虑克隆实验

        Args:
            exp_id (str): 实验CUID
            pro_id (str): 项目CUID
            root_exp_id (str): 根实验CUID
            root_pro_id (str): 根项目CUID
        """
        data = {
            "experimentId": exp_id,
            "projectId": pro_id,
        }
        if root_exp_id and root_pro_id:
            data["rootExperimentId"] = root_exp_id
            data["rootProjectId"] = root_pro_id

        resp = self.http.post("/house/metrics/summaries", data=[data], params={})
        if resp.errmsg:
            return resp

        resp.data = list(resp.data.values())[0]
        resp.data = {
            k: {
                "step": v.get("step"),
                "value": v.get("value"),
                "min": {
                    "step": v.get("min").get("index"),
                    "value": v.get("min").get("data"),
                },
                "max": {
                    "step": v.get("max").get("index"),
                    "value": v.get("max").get("data"),
                }
            }
            for k, v in resp.data.items()
        }
        return resp



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/1 19:40
@File: log_collector.py
@IDE: pycharm
@Description:
    日志集合和上传记录器
"""
import time
from typing import List

from swanlab.error import SyncError
from swanlab.log import swanlog
from swanlab.swanlab_settings import get_settings
from .task_types import UploadType
from .utils import LogQueue
from .utils import ThreadUtil, ThreadTaskABC


class LogCollectorTask(ThreadTaskABC):
    """
    日志聚合器，负责收集所有线程注册的日志信息
    并且定义日志上传接口
    """

    def __init__(self, upload_type=UploadType):
        self.container: List[LogQueue.MsgType] = []
        """
        日志容器，存储从管道中获取的日志信息
        """
        self.lock = False
        self.upload_type = upload_type
        self.upload_interval = get_settings().upload_interval

    @staticmethod
    def report_known_error(errors: List[SyncError]):
        """
        上报错误信息
        :param errors: 错误信息列表
        """
        if len(errors) == 0:
            return
        # 去重
        errors = list(set(errors))
        for error in errors:
            swanlog.__getattribute__(error.log_level)(error.message)

    def upload(self):
        """
        NOTE 此函数运行在其他线程
        上传事件处理
        将收集到的所有上传事件统一触发，上传日志信息
        所有的请求都是网络请求，因此需要异步处理
        对于执行的任务，除了column_type为同步执行，其他为并发执行
        """
        # 根据日志类型进行降重
        upload_tasks_dict = {x: [] for x in self.upload_type}
        # 检查每一个上传结果
        success_tasks_type = []
        # 已知错误列表
        known_errors = []
        # ---------------------------------- 处理upload任务 ----------------------------------

        # 聚合所有的上传任务
        for msg in self.container:
            if msg[0] in self.upload_type:
                upload_tasks_dict[msg[0]].extend(msg[1])

        tasks_key_list = [key for key in upload_tasks_dict if len(upload_tasks_dict[key]) > 0]

        # 同步执行所有的上传任务
        results = [x.value['upload'](upload_tasks_dict[x]) for x in tasks_key_list]
        for index, result in enumerate(results):
            # 如果出现已知问题
            _, e = result
            if isinstance(e, SyncError):
                known_errors.append(e)
                continue
            # 如果出现其他问题，没有办法处理，就直接跳过，但是会有警告
            elif e is not None:
                error = f"{tasks_key_list[index].name} error: {e}, it might be a swanlab bug, data will be lost!"
                swanlog.error(error)
                # continue
                # raise e
            # 标记所有已经成功的任务
            success_tasks_type.append(tasks_key_list[index])

        # ---------------------------------- 最后错误处理 ----------------------------------

        self.container = [(x, upload_tasks_dict[x]) for x in upload_tasks_dict if x not in success_tasks_type]
        self.report_known_error(known_errors)

    def task(self, u: ThreadUtil, *args):
        """
        定时任务，定时上传日志信息
        :param u: 线程工具类
        """
        # 从管道中获取所有的日志信息，存储到self.container中
        if self.lock:
            return swanlog.debug("upload task still in progressing, passed")
        self.container.extend(u.queue.get_all())
        # print("线程" + u.name + "获取到的日志信息: ", self.container)
        if u.timer.can_run(self.upload_interval, len(self.container) == 0):
            self.lock = True
            try:
                self.upload()
            except Exception as e:
                swanlog.error(f"upload error: {e}")
            self.lock = False

    def callback(self, u: ThreadUtil, *args):
        """
        回调函数，用于结束时的回调
        NOTE 此函数运行在主线程
        :param u: 线程工具类
        """
        # 如果当前上传任务正在进行，等待上传任务结束
        while self.lock:
            time.sleep(0.1)
        self.container.extend(u.queue.get_all())
        return self.upload()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/19 14:43
@File: pytest_formater.py
@IDE: pycharm
@Description:

"""
import pytest
from nanoid import generate

# noinspection PyProtectedMember
from swanlab.data.formatter import (
    check_proj_name_format,
    _auto_cut,
    check_key_format,
    check_exp_name_format,
    check_run_id_format,
)


class TestAutoCut:
    @pytest.mark.parametrize(
        "name, value",
        [
            [generate(), generate(size=101)],
            [generate(), generate(size=1000)],
            [generate(), generate(size=10000)],
        ],
    )
    def test_cut(self, name: str, value: str):
        """
        测试自动截断
        """
        assert len(_auto_cut(name, value, 100, True)) == 100

    @pytest.mark.parametrize(
        "name, value",
        [
            [generate(), generate(size=101)],
            [generate(), generate(size=1000)],
            [generate(), generate(size=10000)],
        ],
    )
    def test_no_cut(self, name: str, value: str):
        """
        测试不自动截断
        """
        with pytest.raises(IndexError) as e:
            _auto_cut(name, value, 100, False)
        assert name in str(e.value)


class TestProjName:
    @pytest.mark.parametrize(
        "value",
        [generate(size=100), generate(size=1), "-", "_", ".12", "1", "1.b", "a.b", "+", "1+1"],
    )
    def test_proj_name_common(self, value):
        """
        测试正常情况
        """
        assert check_proj_name_format(value) == value

    @pytest.mark.parametrize("value", [None, 1, [], {}])
    def test_proj_name_type_error(self, value: str):
        """
        测试类型错误
        """
        with pytest.raises(TypeError):
            check_proj_name_format(value)

    @pytest.mark.parametrize("value", ["", " " "啊哈哈", "&^%", "/;]x]"])
    def test_proj_name_value_error(self, value: str):
        """
        测试空值或者不合法值
        """
        with pytest.raises(ValueError):
            check_proj_name_format(value)

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=101),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_proj_name_auto_cut(self, value: str):
        """
        测试自动截断
        """
        assert len(check_proj_name_format(value)) == 100

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=101),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_proj_name_no_cut(self, value: str):
        """
        测试不自动截断
        """
        with pytest.raises(IndexError):
            check_proj_name_format(value, auto_cut=False)


class TestExpName:
    @pytest.mark.parametrize(
        "value",
        [
            generate(size=250),
            generate(size=95),
            generate(size=1),
            "-",
            "_",
            ".12",
            "1",
            "1.b",
            "a.b",
            "+",
            "1+1",
            "你好",
        ],
    )
    def test_exp_name_common(self, value):
        """
        测试正常情况
        """
        assert check_exp_name_format(value) == value

    @pytest.mark.parametrize("value", [None, 1, [], {}])
    def test_exp_name_type_error(self, value: str):
        """
        测试类型错误
        """
        with pytest.raises(TypeError):
            check_exp_name_format(value)

    @pytest.mark.parametrize("value", ["", " "])
    def test_exp_name_value_error(self, value: str):
        """
        测试空值
        """
        with pytest.raises(ValueError):
            check_exp_name_format(value)

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=251),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_exp_name_auto_cut(self, value: str):
        """
        测试自动截断
        """
        assert len(check_exp_name_format(value)) == 250

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=251),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_exp_name_no_cut(self, value: str):
        """
        测试不自动截断
        """
        with pytest.raises(IndexError):
            check_exp_name_format(value, auto_cut=False)


class TestTag:

    @pytest.mark.parametrize(
        "value", [generate(size=255), generate(size=100), generate(size=1), "12", "-", "_", "👾👾👾👾👾👾"]
    )
    def test_tag_common(self, value):
        """
        测试正常情况
        """
        assert check_key_format(value) == value

    def test_key_blank(self):
        """
        测试收尾空格情况
        """
        with pytest.raises(ValueError):
            check_key_format("  ")
        key = "  " + "abc" + "  "
        assert check_key_format(key) == "abc"

    @pytest.mark.parametrize(
        "value",
        [
            None,
            1,
            [],
            {},
        ],
    )
    def test_tag_type_error(self, value: str):
        """
        测试类型错误
        """
        with pytest.raises(TypeError):
            check_key_format(value)

    @pytest.mark.parametrize("value", ["", "   ", " " * 256, ".sas", "/asa", "abc/", "bac."])
    def test_tag_value_error(self, value: str):
        """
        测试不合法值
        """
        with pytest.raises(ValueError):
            check_key_format(value)

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=256),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_tag_auto_cut(self, value: str):
        """
        测试自动截断
        """
        assert len(check_key_format(value)) == 255

    @pytest.mark.parametrize(
        "value",
        [
            generate(size=256),
            generate(size=1000),
            generate(size=10000),
        ],
    )
    def test_tag_no_cut(self, value: str):
        """
        测试不自动截断
        """
        with pytest.raises(IndexError):
            check_key_format(value, auto_cut=False)


class TestRunIdFormat:
    @staticmethod
    def test_run_id_format_valid_run_id():
        assert check_run_id_format("abcdefghijklmnopqrstu") == "abcdefghijklmnopqrstu"

    @staticmethod
    def test_run_id_format_invalid_length():
        with pytest.raises(
            ValueError, match=r"id .* is invalid, it must be 21 characters of lowercase letters and digits"
        ):
            check_run_id_format("shortid")

    @staticmethod
    def test_run_id_format_invalid_characters():
        with pytest.raises(
            ValueError, match=r"id .* is invalid, it must be 21 characters of lowercase letters and digits"
        ):
            check_run_id_format("abc123!@#def456ghi789")

    @staticmethod
    def test_run_id_format_none_input():
        assert check_run_id_format(None) is None

    @staticmethod
    def test_run_id_format_numeric_input():
        assert check_run_id_format(123456789012345678901) == "123456789012345678901"

    @staticmethod
    def test_run_id_format_empty_string():
        with pytest.raises(
            ValueError, match=r"id .* is invalid, it must be 21 characters of lowercase letters and digits"
        ):
            check_run_id_format("")



from typing import Dict, Optional

from swanlab.data.modules import DataWrapper
from swanlab.log import swanlog
from swanlab.toolkit import (
    MetricInfo,
    MetricErrorInfo,
    ColumnClass,
    SectionType,
    ColumnConfig,
)
from .helper import SwanLabRunOperator
from .key import SwanLabKey
from ..store import get_run_store


class SwanLabExp:
    """
    Class for running experiments
    save keys when running experiments
    """

    def __init__(self, operator: SwanLabRunOperator) -> None:
        """初始化实验

        Parameters
        ----------
        operator : SwanLabRunOperator
            操作员
        """
        self._run_store = get_run_store()
        self._operator = operator
        # 当前实验的所有tag数据字段
        self._keys: Dict[str, SwanLabKey] = {}
        # 恢复实验时，同步云端实验的指标数据
        if self._run_store.metrics is not None:
            media_dir, log_dir = self._run_store.media_dir, self._run_store.log_dir
            for kid, (key, (column_type, column_class, error, step)) in enumerate(self._run_store.metrics.items()):
                key_index = self._generate_key_index(key, column_class)
                self._keys[key_index], column_info = SwanLabKey.mock_from_remote(
                    key, column_type, column_class, error, media_dir, log_dir, kid, step
                )
                self._operator.on_column_create(column_info)

    @staticmethod
    def _generate_key_index(key: str, column_class: ColumnClass) -> str:
        """
        生成key的索引
        :param key: key的云端标识
        :param column_class: 列类型，CUSTOM为自定义key，SYSTEM为系统key
        """
        if column_class not in ('CUSTOM', 'SYSTEM'):
            raise RuntimeError(
                f"Column class must be 'CUSTOM' or 'SYSTEM', got {column_class}. "
                f"Maybe you need to update swanlab: pip install -U swanlab"
            )
        return f"{column_class}-{key}"

    def _warn_type_error(self, key_index: str, key: str):
        """警告类型错误
        执行此方法时需保证key已经存在
        """
        key_obj = self._keys[key_index]
        class_name = key_obj.column_info.got
        expected = key_obj.column_info.expected
        if key_obj.is_chart_valid:
            return
        if class_name == "list":
            swanlog.error(f"Data type error, key: {key}, there is element of invalid data type in the list.")
        else:
            swanlog.error(f"Data type error, key: {key}, data type: {class_name}, expected: {expected}")

    def _warn_chart_error(self, key_index: str, key: str):
        """
        警告图表创建错误
        执行此方法时需保证key已经存在
        """
        key_obj = self._keys[key_index]
        if key_obj.is_chart_valid:
            return
        class_name = key_obj.column_info.got
        if class_name == "list":
            swanlog.warning(
                f"Chart '{key}' creation failed. "
                f"Reason: The data type in list of the key '{key}' is not as expected, please check the data type."
            )
        else:
            swanlog.warning(
                f"Chart '{key}' creation failed. "
                f"Reason: The expected value type for the chart '{key}' is one of int,"
                f"float or BaseType, but the input type is {class_name}."
            )

    def _add(
        self,
        key: str,
        name: Optional[str],
        column_class: ColumnClass,
        column_config: Optional[ColumnConfig],
        section_type: SectionType,
        data: DataWrapper,
        step: int = None,
    ) -> MetricInfo:
        """记录一条新的key数据

        Parameters
        ----------
        key : str
            key的云端唯一标识
        name : str
            key的实际名称
        column_class : str
            列类型，CUSTOM为自定义key，SYSTEM为系统key
        section_type : str
            key的组类型
        data : DataWrapper
            包装后的数据，用于数据解析
        step : int, optional
            步数，如果不传则默认当前步数为'已添加数据数量+1'
            在log函数中已经做了处理，此处不需要考虑数值类型等情况
        """
        key_index = self._generate_key_index(key, column_class)
        # 判断tag是否存在，如果不存在则创建tag
        key_obj: SwanLabKey = self._keys.get(key_index, None)

        # ---------------------------------- 包装器解析 ----------------------------------

        if step is not None and not isinstance(step, int):
            swanlog.warning(f"Step {step} is not int, SwanLab will set it automatically.")
            step = None
        if key_obj is None:
            step = 0 if step is None or not isinstance(step, int) else step
        else:
            step = len(key_obj.steps) if step is None else step
            if step in key_obj.steps:
                swanlog.debug(f"Step {step} on key {key} already exists, ignored.")
                return MetricErrorInfo(column_info=key_obj.column_info, error=DataWrapper.create_duplicate_error())
        data.parse(step=step, key=key)

        # ---------------------------------- 图表创建 ----------------------------------

        if key_obj is None:
            num = len(self._keys)
            # 将此tag对象添加到实验列表中
            key_obj = SwanLabKey(key, self._run_store.media_dir, self._run_store.log_dir)
            self._keys[key_index] = key_obj
            # 新建图表，完成数据格式校验
            column_info = key_obj.create_column(
                key,
                name,
                column_class,
                column_config,
                section_type,
                data,
                num,
            )
            self._warn_type_error(key_index, key)
            # 创建新列，生成回调
            self._operator.on_column_create(column_info)

        # 检查tag创建时图表是否创建成功，如果失败则也没有写入数据的必要了，直接退出
        if not key_obj.is_chart_valid:
            self._warn_chart_error(key_index, key)
            return MetricErrorInfo(key_obj.column_info, error=key_obj.column_info.error)
        key_info = key_obj.add(data)
        key_info.buffers = data.parse().buffers
        key_info.media_dir = self._run_store.media_dir
        return key_info

    def add(
        self,
        data: DataWrapper,
        key: str,
        name: str = None,
        column_class: ColumnClass = 'CUSTOM',
        column_config: Optional[ColumnConfig] = None,
        section_type: SectionType = "PUBLIC",
        step: int = None,
    ) -> MetricInfo:
        """记录一条新的key数据
        Parameters
        ----------
        data : DataWrapper
            包装后的数据，用于数据解析
        key : str
            列的云端唯一标识
        name : str
            列的实际名称, 默认与key相同
        column_class : str, optional
            列的类型
        column_config : Optional[ColumnConfig], optional
            列的额外配置信息
        section_type : str, optional
            key的组类型
        step : int, optional
            步数，如果不传则默认当前步数为'已添加数据数量+1'
            在log函数中已经做了处理，此处不需要考虑数值类型等情况
        """
        section_type: SectionType = 'CUSTOM' if data.is_custom else section_type
        m = self._add(key, name, column_class, column_config, section_type, data, step)
        self._operator.on_metric_create(m)
        return m



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/1 16:42
@File: __init__.py
@IDE: pycharm
@Description:
    云端日志资源上传部分
    NOTE: 这部分设计已经过时，后续考虑优化
"""
from .start_thread import ThreadPool
from .task_types import UploadType

__all__ = ["UploadType", "ThreadPool"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/7/18 15:28
@File: test_env.py
@IDE: pycharm
@Description:
    测试swanlab.env模块
"""
import os

import pytest

import swanlab
from swanlab.env import SwanLabEnv, is_interactive, get_save_dir


def test_set_default():
    """
    测试获取默认的环境变量
    """
    del os.environ[SwanLabEnv.WEB_HOST.value]
    del os.environ[SwanLabEnv.API_HOST.value]
    del os.environ[SwanLabEnv.RUNTIME.value]
    swanlab.env.SwanLabEnv.set_default()
    assert swanlab.package.get_host_web() == "https://swanlab.cn"
    assert swanlab.package.get_host_api() == "https://api.swanlab.cn/api"
    assert os.getenv(SwanLabEnv.RUNTIME.value) == "user"


def test_set_by_netrc():
    """
    测试通过netrc文件设置环境变量
    """
    del os.environ[SwanLabEnv.WEB_HOST.value]
    del os.environ[SwanLabEnv.API_HOST.value]
    del os.environ[SwanLabEnv.RUNTIME.value]
    netrc_path = os.path.join(get_save_dir(), ".netrc")
    with open(netrc_path, "w") as f:
        f.write("machine https://example.cn\nlogin test\npassword 123")
    swanlab.env.SwanLabEnv.set_default()
    assert swanlab.package.get_host_web() == "https://example.cn"
    assert swanlab.package.get_host_api() == "https://example.cn/api"
    assert os.getenv(SwanLabEnv.RUNTIME.value) == "user"


def test_check():
    """
    测试检查环境变量
    """
    os.environ[SwanLabEnv.MODE.value] = "124345"
    with pytest.raises(ValueError):
        SwanLabEnv.check()
    os.environ[SwanLabEnv.RUNTIME.value] = "124"
    with pytest.raises(ValueError):
        SwanLabEnv.check()


def test_is_interactive():
    # 测试时默认返回true
    assert is_interactive() == True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/13 01:35
@File: watch.py
@IDE: pycharm
@Description:
    watch命令
"""
import os
import socket
import sys

import click

from swanlab.env import get_swanlog_dir, SwanLabEnv
from swanlab.log import swanlog


def get_free_port(address='0.0.0.0', default_port=5092) -> int:
    """
    获取一个可用端口
    NOTE: 默认情况下，返回5092端口，如果端口被占用，返回一个随机可用端口
    WARNING: 不能保证独占,极稀有情况下两个程序占用到此端口
    ---
    Args:
        address: 主机(host)地址
        default_port: 默认端口号

    Return:
        port: 一个可用端口
    """
    # 判断是否占用
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        try:
            s.bind((address, default_port))
        except OSError:
            pass
        else:
            return default_port
    # 如果占用就返回一个随机可用端口
    sock = socket.socket()
    sock.bind((address, 0))
    ip, port = sock.getsockname()
    sock.close()
    return port


@click.command()
@click.argument(
    "path",
    envvar=SwanLabEnv.SWANLOG_FOLDER.value,
    type=click.Path(
        exists=True,
        dir_okay=True,
        file_okay=False,
        resolve_path=True,
        readable=True,
    ),
    nargs=1,
    required=False,
)
@click.option(
    "--host",
    "-h",
    default=lambda: os.environ.get(SwanLabEnv.SWANBOARD_HOST.value, "127.0.0.1"),
    type=str,
    nargs=1,
    help="The host of swanlab web, default by 127.0.0.1",
)
@click.option(
    "--port",
    "-p",
    default=lambda: os.environ.get(SwanLabEnv.SWANBOARD_PROT.value, get_free_port()),
    nargs=1,
    type=click.IntRange(1, 65535),
    help="The port of swanlab web, default by 5092",
)
@click.option(
    "--logdir",
    "-l",
    default=lambda: os.environ.get(SwanLabEnv.SWANLOG_FOLDER.value, None),
    nargs=1,
    type=click.Path(
        exists=True,
        dir_okay=True,
        file_okay=False,
        resolve_path=True,
        readable=True,
    ),
    help="Specify the folder to store Swanlog, which is by default the folder where Swanlab Watch is run."
    "The option will be deprecated in the future, you can just use `swanlab watch <LOG PATH>` to specify the path.",
)
@click.option(
    "--log-level",
    default="info",
    nargs=1,
    type=click.Choice(["debug", "info", "warning", "error", "critical"]),
    help="The level of log, default by info; You can choose one of [debug, info, warning, error, critical]",
)
def watch(path: str, host: str, port: int, logdir: str, log_level: str):
    """
    Run this commands to turn on the swanlab service.
    """
    try:
        # noinspection PyPackageRequirements
        from swanboard import SwanBoardRun
    except ModuleNotFoundError:
        click.echo("Please install the swanboard package: `pip install swanlab[dashboard]`")
        return sys.exit(1)
    swanlog.level = log_level
    # ----- 校验path，path如果被输入，已经由上层校验已存在，可读，是一个文件夹 -----
    if logdir is not None:
        swanlog.warning(
            "The option `--logdir` will be deprecated in the future, "
            f"you can just use `swanlab watch [PATH]` to specify the path."
        )
    # logdir 覆盖 path，接下来统一处理path而不再管logdir
    path = logdir if logdir is not None else path
    if path is not None:
        path = os.path.abspath(path)
        os.environ[SwanLabEnv.SWANLOG_FOLDER.value] = path
    # 为None时从环境变量中获取
    try:
        path = get_swanlog_dir()
        # 产品经理要求无论日志文件夹是否存在都不报错 🤡 ，给用户以开启web服务的“爽感”
        # if not os.path.exists(path):
        #     raise FileNotFoundError
    except ValueError as e:
        click.BadParameter(str(e))
        return sys.exit(3)
    except NotADirectoryError:
        click.BadParameter("SWANLAB_LOG_DIR must be a directory")
        return sys.exit(4)
    except FileNotFoundError:
        click.BadParameter(f"The log folder `{path}` was not found")
        return sys.exit(5)
    # ----- 校验host和port -----
    try:
        SwanBoardRun.is_valid_port(port)
        SwanBoardRun.is_valid_ip(host)
    except ValueError as e:
        click.BadParameter(str(e))
        return sys.exit(6)
    # ---- 启动服务 ----
    SwanBoardRun.run(
        path=path,
        host=host,
        port=port,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 01:26
@File: test_exp.py
@IDE: pycharm
@Description:
    测试运行实验类
"""


def test_exp_add_ok():
    pass


def test_exp_add_wrong_type():
    pass



"""
@author: cunyue
@file: test_models.py
@time: 2025/6/8 14:16
@description: 测试备份模型
"""

import json
import os

import pytest
import yaml

from swanlab.log.type import LogData, LogContent
from swanlab.proto.v0 import BaseModel, Log, Runtime
from swanlab.toolkit import create_time
from tutils import TEMP_PATH


def test_base():
    base_model = BaseModel()
    assert base_model.to_record() == '{"model_type": "BaseModel", "data": {}}\n'
    # BaseModel 本身无法使用 from_record 方法进行反序列化
    with pytest.raises(ValueError):
        assert base_model.from_record('{"model_type": "BaseModel", "data": {}}\n') is not None
    with pytest.raises(ValueError):
        base_model.from_record('{"model_type": "WrongType", "data": {}\n')


@pytest.mark.parametrize(
    "log_type",
    [
        ["stdout", "INFO"],
        ["stderr", "WARN"],
    ],
)
def test_log(log_type):
    ct = create_time()
    log_data = LogData(type=log_type[0], contents=[LogContent(message="Test log message", create_time=ct, epoch=1)])
    logs = Log.from_log_data(log_data=log_data)
    assert len(logs) == 1
    log_model = logs[0].to_log_model()
    assert log_model['level'] == log_type[1]
    assert len(log_model['contents']) == 1
    log_content = log_model['contents'][0]
    assert log_content['message'] == "Test log message"
    assert log_content['epoch'] == 1
    assert log_content['create_time'] == ct


@pytest.mark.parametrize("conda", [None, "test conda"])
@pytest.mark.parametrize("requirements", [None, "test requirements"])
@pytest.mark.parametrize("metadata", [None, {"key": "value"}])
@pytest.mark.parametrize("config", [None, {"setting": "value"}])
def test_runtime(conda, requirements, metadata, config):
    from swanlab.toolkit import RuntimeInfo

    # 1. runtime info -> runtime
    runtime_info = RuntimeInfo(requirements, metadata, config, conda)
    runtime = Runtime.from_runtime_info(runtime_info)
    if conda is not None:
        assert runtime.conda_filename == runtime_info.conda.name
    else:
        assert runtime.conda_filename is None
    if requirements is not None:
        assert runtime.requirements_filename == runtime_info.requirements.name
    else:
        assert runtime.requirements_filename is None
    if metadata is not None:
        assert runtime.metadata_filename == runtime_info.metadata.name
    else:
        assert runtime.metadata_filename is None
    if config is not None:
        assert runtime.config_filename == runtime_info.config.name
    else:
        assert runtime.config_filename is None
    # 2. runtime -> file model
    #  模拟文件写入
    if runtime.conda_filename is not None:
        with open(os.path.join(TEMP_PATH, runtime.conda_filename), 'w') as f:
            f.write(conda)
    if runtime.requirements_filename is not None:
        with open(os.path.join(TEMP_PATH, runtime.requirements_filename), 'w') as f:
            f.write(requirements)
    # 这里只检查 metadata 和 config 是否被正确写入和读取，不检查内容
    if runtime.metadata_filename is not None:
        with open(os.path.join(TEMP_PATH, runtime.metadata_filename), 'w') as f:
            json.dump(metadata, f)  # noqa: Expected type 'SupportsWrite[str]', got 'TextIO' instead
    if runtime.config_filename is not None:
        with open(os.path.join(TEMP_PATH, runtime.config_filename), 'w') as f:
            yaml.safe_dump(config, f)

    file_model = runtime.to_file_model(TEMP_PATH)
    if conda is not None:
        assert file_model.conda == runtime_info.conda.info, "Conda file name mismatch"
    else:
        assert file_model.conda is None
    if requirements is not None:
        assert file_model.requirements == runtime_info.requirements.info, "Requirements file name mismatch"
    else:
        assert file_model.requirements is None
    if metadata is not None:
        assert str(file_model.metadata) == str(runtime_info.metadata.info), "Metadata file name mismatch"
    else:
        assert file_model.metadata is None
    if config is not None:
        assert str(file_model.config) == str(runtime_info.config.info), "Config file name mismatch"
    else:
        assert file_model.config is None


# 其他类似 但是感觉没必要写



"""
@author: zeyi-lin
@file: metax.py
@time: 2025-05-31 08:29:00
@description: Metax GPU信息采集
"""

import math
import platform
import subprocess
from typing import Optional, Tuple

from ..type import HardwareCollector as H
from ..type import HardwareConfig, HardwareFuncResult, HardwareInfoList
from ..utils import generate_key, random_index


def get_metax_gpu_info() -> HardwareFuncResult:
    """
    获取MetaX GPU信息，包括驱动版本、设备信息等
    """
    # MetaX GPU只支持Linux系统
    if platform.system() != "Linux":
        return None, None

    info = {"driver": None, "maca": None, "gpu": None}
    collector = None
    try:
        driver, maca_version, gpu_map = map_metax_gpu()
        info["driver"] = driver
        info["maca"] = maca_version
        info["gpu"] = gpu_map
        max_mem_value: int = 0
        for gpu_id in gpu_map:
            mem_value = gpu_map[gpu_id]["memory"]
            if mem_value > max_mem_value:
                max_mem_value = mem_value
        max_mem_value = max_mem_value * 1024
        collector = MetaxCollector(gpu_map, max_mem_value)
    except Exception:  # noqa
        if all(v is None for v in info.values()):
            return None, None
    return info, collector


def map_metax_gpu() -> Tuple[Optional[str], Optional[str], dict]:
    """
    获取 metax gpu 信息，包括驱动版本、设备信息等，例如：
    driver: '2.1.12'
    gpu_map: {"0": { "name": "MetaX C500”, "memory": "64GB"}, "1": { "name": "MetaX C500", "memory": "64GB"}, ...}
    """
    output_str = subprocess.run(["mx-smi"], capture_output=True, check=True, text=True).stdout

    driver = None
    maca_version = None
    gpu_map = {}
    index = 0

    output_str_line = output_str.split("\n")

    for line in output_str_line:
        if "mx-smi" in line:
            driver = line.split(" ")[-1]
        if "MACA" in line:
            maca_version = line.split(" ")[3]
        elif "MetaX" in line and "Management" not in line:
            gpu_map[index] = {}
            line_split = line.split(" ")
            line_split = [item for item in line_split if item != ""]
            gpu_map[index]["name"] = line_split[3]
        elif "MiB" in line:
            try:
                line_split = line.split(" ")
                line_split = [item for item in line_split if item != ""]
                gpu_memory = int(line_split[-4].split("/")[-1]) // 1024
                gpu_map[index]["memory"] = gpu_memory
                index += 1
            except Exception as e:
                pass

    return driver, maca_version, gpu_map


class MetaxCollector(H):
    def __init__(self, gpu_map, max_mem_value):
        super().__init__()
        self.gpu_map = gpu_map
        self.max_mem_value = max_mem_value

        # GPU Utilization (%)
        self.util_key = generate_key("gpu.{gpu_index}.pct")
        util_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="GPU Utilization (%)",
        )
        self.per_util_configs = {}

        # GPU Memory Allocated (%)
        self.memory_key = generate_key("gpu.{gpu_index}.mem.pct")
        memory_config = HardwareConfig(
            y_range=(0, 100),
            chart_index=random_index(),
            chart_name="GPU Memory Allocated (%)",
        )
        self.per_memory_configs = {}

        # GPU Memory Allocated (MB)
        self.memory_value_key = generate_key("gpu.{gpu_index}.mem.value")
        memory_value_config = HardwareConfig(
            y_range=(0, self.max_mem_value),
            chart_index=random_index(),
            chart_name="GPU Memory Allocated (MB)",
        )
        self.per_memory_value_configs = {}

        # GPU Temperature (°C)
        self.temp_key = generate_key("gpu.{gpu_index}.temp")
        temp_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="GPU Temperature (°C)",
        )
        self.per_temp_configs = {}

        # GPU Power (W)
        self.power_key = generate_key("gpu.{gpu_index}.power")
        power_config = HardwareConfig(
            y_range=(0, None),
            chart_index=random_index(),
            chart_name="GPU Power (W)",
        )
        self.per_power_configs = {}

        for gpu_id in self.gpu_map:
            metric_name = f"GPU {gpu_id}"
            self.per_util_configs[metric_name] = util_config.clone(metric_name=metric_name)
            self.per_memory_configs[metric_name] = memory_config.clone(metric_name=metric_name)
            self.per_memory_value_configs[metric_name] = memory_value_config.clone(metric_name=metric_name)
            self.per_temp_configs[metric_name] = temp_config.clone(metric_name=metric_name)
            self.per_power_configs[metric_name] = power_config.clone(metric_name=metric_name)

    def collect(self) -> HardwareInfoList:
        result: HardwareInfoList = []
        usage_methods = [
            self.get_utilization_usage,
            self.get_memory_rate_usage,
            self.get_memory_value_usage,
            self.get_temperature_usage,
            self.get_power_usage,
        ]

        for method in usage_methods:
            result.extend(method().values())
        return result

    def get_utilization_usage(self) -> dict:
        """
        获取指定GPU设备的利用率
        """
        output_str = subprocess.run(["mx-smi", "--show-usage"], capture_output=True, check=True, text=True).stdout

        output_str_line = output_str.split("\n")
        usage_infos = {}
        index = 0

        for line in output_str_line:
            if "GPU" in line and "%" in line:
                gpu_id = index
                usage_line = line.split(" ")
                usage_line = [item for item in usage_line if item != ""]
                gpu_usage = float(usage_line[-2])

                usage_infos[gpu_id] = {
                    "key": self.util_key.format(gpu_index=gpu_id),
                    "name": f"GPU {gpu_id} Utilization (%)",
                    "value": math.nan,
                    "config": self.per_util_configs[f"GPU {gpu_id}"],
                }

                usage_infos[gpu_id]["value"] = gpu_usage
                index += 1

        return usage_infos

    def get_memory_rate_usage(self) -> dict:
        """
        获取指定GPU设备的内存占用率
        """
        output_str = subprocess.run(
            ["mx-smi", "--show-memory"],
            capture_output=True,
            text=True,
        ).stdout
        output_str_line = output_str.split("\n")
        mem_infos = {}
        index = 0
        for line in output_str_line:
            if "vis_vram total" in line:
                gpu_id = index
                gpu_mem_total = line.split(" ")[-2]
            if "vis_vram used" in line:
                gpu_id = index
                gpu_mem_used = line.split(" ")[-2]

                mem_infos[gpu_id] = {
                    "key": self.memory_key.format(gpu_index=gpu_id),
                    "name": f"GPU {gpu_id} Memory Allocated (%)",
                    "value": math.nan,
                    "config": self.per_memory_configs[f"GPU {gpu_id}"],
                }

                gpu_mem_usage_rate = float(gpu_mem_used) / float(gpu_mem_total) * 100
                mem_infos[gpu_id]["value"] = gpu_mem_usage_rate
                index += 1

        return mem_infos

    def get_memory_value_usage(self) -> dict:
        """
        获取指定GPU设备的内存占用值
        """
        output_str = subprocess.run(
            ["mx-smi", "--show-memory"],
            capture_output=True,
            text=True,
        ).stdout
        output_str_line = output_str.split("\n")
        mem_value_infos = {}
        index = 0
        for line in output_str_line:
            if "vis_vram used" in line:
                gpu_id = index
                gpu_mem_used = line.split(" ")[-2]
                mem_value_infos[gpu_id] = {
                    "key": self.memory_value_key.format(gpu_index=gpu_id),
                    "name": f"GPU {gpu_id} Memory Allocated (MB)",
                    "value": math.nan,
                    "config": self.per_memory_value_configs[f"GPU {gpu_id}"],
                }
                gpu_mem_usage_value = float(gpu_mem_used) / 1024
                mem_value_infos[gpu_id]["value"] = gpu_mem_usage_value
                index += 1
        return mem_value_infos

    def get_temperature_usage(self) -> dict:
        """
        获取指定GPU设备的利用率
        """
        output_str = subprocess.run(["mx-smi", "--show-temperature"], capture_output=True, check=True, text=True).stdout

        output_str_line = output_str.split("\n")
        temp_infos = {}
        index = 0

        for line in output_str_line:
            if "hotspot" in line:
                gpu_id = index
                gpu_temp = line.split(" ")[-2]

                temp_infos[gpu_id] = {
                    "key": self.temp_key.format(gpu_index=gpu_id),
                    "name": f"GPU {gpu_id} Temperature (°C)",
                    "value": math.nan,
                    "config": self.per_temp_configs[f"GPU {gpu_id}"],
                }
                temp_infos[gpu_id]["value"] = float(gpu_temp)
                index += 1

        return temp_infos

    def get_power_usage(self) -> dict:
        """
        获取指定GPU设备的功耗(W)
        """
        output_str = subprocess.run(["mx-smi", "--show-board-power"], capture_output=True, check=True, text=True).stdout

        output_str_line = output_str.split("\n")
        power_infos = {}
        index = 0

        for line in output_str_line:
            if "Total" in line and "W" in line:
                total_line = line.split(" ")
                total_line = [item for item in total_line if item != ""]
                total_power = float(total_line[-2])
                power_infos[index] = {
                    "key": self.power_key.format(gpu_index=index),
                    "name": f"GPU {index} Power (W)",
                    "value": math.nan,
                    "config": self.per_power_configs[f"GPU {index}"],
                }
                power_infos[index]["value"] = total_power
                index += 1

        return power_infos



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/6/3 02:34
@File: __init__.py
@IDE: pycharm
@Description:
    音频模块
"""
from typing import Union, Any

from swanlab.toolkit import MediaBuffer, DataSuite as D, MediaType

try:
    # noinspection PyPackageRequirements
    import soundfile as sf

    # noinspection PyPackageRequirements
    import numpy as np

    InputType = Union[str, np.ndarray]
except ImportError:
    sf, np = None, None
    InputType = Union[str, Any]


class Audio(MediaType):
    SF_SUPPORT_DTYPE = [np.dtype(d) for d in ["float32", "float64", "int16", "int32"]] if sf else []

    def __init__(self, data_or_path: InputType, sample_rate: int = 44100, caption: str = None):
        """Audio class constructor

        Parameters
        ----------
        data_or_path: str or numpy.ndarray
            Path to an audio file or numpy array of audio data.
        sample_rate: int
                Sample rate of the audio data. Required when input is numpy array.
        caption: str
            Caption for the audio.
        """
        if sf is None or np is None:
            raise ImportError(
                "soundfile and numpy are required for Audio class, "
                "you can install them by `pip install soundfile numpy`"
            )
        super().__init__()
        if isinstance(data_or_path, str):
            # 如果输入为路径字符串
            try:
                audio_data, sample_rate = sf.read(data_or_path)
                # 转置为 (num_channels, num_frames) 的形式
                audio_data = audio_data.T
            except Exception as e:
                raise ValueError(f"Invalid audio path: {data_or_path}") from e
        elif isinstance(data_or_path, np.ndarray):
            # 如果输入为numpy array ，要求输入为 (num_channels, num_frames) 的形式
            # 支持单声道 或 双声道 两种形式

            if data_or_path.dtype not in self.SF_SUPPORT_DTYPE:
                e = (
                    f"Invalid numpy array for the audio data, support dtype is {self.SF_SUPPORT_DTYPE}, "
                    f"but got {data_or_path.dtype}"
                )
                raise TypeError(e)

            # 如果data_or_path是一维, 则reshape为2维
            if len(data_or_path.shape) == 1:
                data_or_path = data_or_path.reshape(1, -1)

            # 获取通道数
            num_channels = data_or_path.shape[0]

            if num_channels != 2 and num_channels != 1:
                raise TypeError("Invalid numpy array for the audio data, support shape is (num_channels, num_frames)")
            if sample_rate is None:
                raise TypeError("sample_rate must be provided when input is numpy array while constructing Audio()")
            audio_data = data_or_path
        else:
            # 以上都不是，则报错
            raise TypeError("Unsupported audio type. Please provide a valid path or numpy array.")
        self.audio_data = audio_data
        """
        转换为矩阵后的数据
        """
        self.buffer = MediaBuffer()
        sf.write(self.buffer, audio_data.T, sample_rate, format="wav")
        self.sample_rate = sample_rate
        self.caption = D.check_caption(caption)

    # ---------------------------------- 覆写方法 ----------------------------------
    def parse(self):
        # 文件名称
        hash_name = D.get_hash_by_ndarray(self.audio_data)[:16]
        save_name = f"audio-step{self.step}-{hash_name}.wav"
        return save_name, self.buffer

    def get_more(self):
        """返回more数据"""
        # 如果传入的是Audio类列表
        return {"caption": self.caption} if self.caption is not None else None

    def get_section(self):
        """设定分组名"""
        return "Audio"

    def get_chart(self):
        """设定图表类型"""
        return self.Chart.AUDIO



"""
// @author: ComPleHN
// @file: feo.vue
// @time: 2025/5/27 15:53
// @description: 本文件是对于echarts的 地理坐标 图表测试
"""
# ---------------------------------------------- Geo - Geo_chart_countries_js ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.datasets import register_url

# try:
#     register_url("https://assets.pyecharts.org/assets/maps/")
# except Exception:
#     import ssl
#
#     ssl._create_default_https_context = ssl._create_unverified_context
#     register_url("https://assets.pyecharts.org/assets/maps/")

c1 = (
    Geo()
    .add_schema(maptype="瑞士")
    .set_global_opts(title_opts=opts.TitleOpts(title="瑞士"))
)

# ---------------------------------------------- Geo - Geo_lines_background ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.globals import ChartType, SymbolType

c2 = (
    Geo()
    .add_schema(
        maptype="china",
        itemstyle_opts=opts.ItemStyleOpts(color="#323c48", border_color="#111"),
    )
    .add(
        "",
        [("广州", 55), ("北京", 66), ("杭州", 77), ("重庆", 88)],
        type_=ChartType.EFFECT_SCATTER,
        color="white",
    )
    .add(
        "geo",
        [("广州", "上海"), ("广州", "北京"), ("广州", "杭州"), ("广州", "重庆")],
        type_=ChartType.LINES,
        effect_opts=opts.EffectOpts(
            symbol=SymbolType.ARROW, symbol_size=6, color="blue"
        ),
        linestyle_opts=opts.LineStyleOpts(curve=0.2),
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(title_opts=opts.TitleOpts(title="Geo-Lines-background"))
)

# ---------------------------------------------- Geo - Geo_visualmap_piecewise ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.faker import Faker

c3 = (
    Geo()
    .add_schema(maptype="china")
    .add("geo", [list(z) for z in zip(Faker.provinces, Faker.values())])
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(is_piecewise=True),
        title_opts=opts.TitleOpts(title="Geo-VisualMap（分段型）"),
    )
)

# ---------------------------------------------- Geo - Geo_lines ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.globals import ChartType, SymbolType

c4 = (
    Geo()
    .add_schema(maptype="china")
    .add(
        "",
        [("广州", 55), ("北京", 66), ("杭州", 77), ("重庆", 88)],
        type_=ChartType.EFFECT_SCATTER,
        color="white",
    )
    .add(
        "geo",
        [("广州", "上海"), ("广州", "北京"), ("广州", "杭州"), ("广州", "重庆")],
        type_=ChartType.LINES,
        effect_opts=opts.EffectOpts(
            symbol=SymbolType.ARROW, symbol_size=6, color="blue"
        ),
        linestyle_opts=opts.LineStyleOpts(curve=0.2),
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(title_opts=opts.TitleOpts(title="Geo-Lines"))
)

# ---------------------------------------------- Geo - Geo_guangdong ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.faker import Faker
from pyecharts.globals import ChartType

c5 = (
    Geo()
    .add_schema(maptype="广东")
    .add(
        "geo",
        [list(z) for z in zip(Faker.guangdong_city, Faker.values())],
        type_=ChartType.HEATMAP,
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(), title_opts=opts.TitleOpts(title="Geo-广东地图")
    )
)

# ---------------------------------------------- Geo - Geo_heatmap ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.faker import Faker
from pyecharts.globals import ChartType

c6 = (
    Geo()
    .add_schema(maptype="china")
    .add(
        "geo",
        [list(z) for z in zip(Faker.provinces, Faker.values())],
        type_=ChartType.HEATMAP,
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(),
        title_opts=opts.TitleOpts(title="Geo-HeatMap"),
    )
)

# ---------------------------------------------- Geo - Geo_effectscatter ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.faker import Faker
from pyecharts.globals import ChartType

c7 = (
    Geo()
    .add_schema(maptype="china")
    .add(
        "geo",
        [list(z) for z in zip(Faker.provinces, Faker.values())],
        type_=ChartType.EFFECT_SCATTER,
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(title_opts=opts.TitleOpts(title="Geo-EffectScatter"))
)

# ---------------------------------------------- Geo - Geo_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.faker import Faker

c8 = (
    Geo()
    .add_schema(maptype="china")
    .add("geo", [list(z) for z in zip(Faker.provinces, Faker.values())])
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(), title_opts=opts.TitleOpts(title="Geo-基本示例")
    )
)

# ---------------------------------------------- Geo - Geo_echart_china_js ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Geo
from pyecharts.datasets import register_url

# try:
#     register_url("https://assets.pyecharts.org/assets/v5/maps/")
# except Exception:
#     import ssl
#
#     ssl._create_default_https_context = ssl._create_unverified_context
#     register_url("https://assets.pyecharts.org/assets/v5/maps/")

c9 = (
    Geo()
    .add_schema(maptype="海淀区")
    .set_global_opts(title_opts=opts.TitleOpts(title="海淀区"))
)




import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="geo",
    public=True,
)

swanlab.log(
    {
        "Geo - Geo_chart_countries_js": c1,
        "Geo - Geo_lines_background": c2,
        "Geo - Geo_visualmap_piecewise": c3,
        "Geo - Geo_lines": c4,
        "Geo - Geo_guangdong": c5,
        "Geo - Geo_heatmap": c6,
        "Geo - Geo_effectscatter": c7,
        "Geo - Geo_base": c8,
        "Geo - Geo_echart_china_js": c9
    }
)



from .notification import EmailCallback, LarkCallback, PrintCallback, SlackCallback, DingTalkCallback, WXWorkCallback, DiscordCallback
from .writer import CSVWriter, LogdirFileWriter

__all__ = ["EmailCallback", "LarkCallback", "PrintCallback", "CSVWriter", "WebhookCallback", "DingTalkCallback", "WXWorkCallback", "DiscordCallback", "SlackCallback", "LogdirFileWriter"]



"""
// @author: ComPleHN
// @file: liquid.vue
// @time: 2025/5/27 18:45
// @description: 本文件是对于echarts的 水球图 图表测试 
"""
# ---------------------------------------------- Liquid - Liquid_shape_diamond ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid
from pyecharts.globals import SymbolType

c1 = (
    Liquid()
    .add("lq", [0.3, 0.7], is_outline_show=False, shape=SymbolType.DIAMOND)
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-Shape-Diamond"))
)

# ---------------------------------------------- Liquid - Multiple_liquid ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Grid, Liquid
from pyecharts.commons.utils import JsCode

l1 = (
    Liquid()
    .add("lq", [0.6, 0.7], center=["60%", "50%"])
    .set_global_opts(title_opts=opts.TitleOpts(title="多个 Liquid 显示"))
)

l2 = Liquid().add(
    "lq",
    [0.3254],
    center=["25%", "50%"],
    label_opts=opts.LabelOpts(
        font_size=50,
        formatter=JsCode(
            """function (param) {
                    return (Math.floor(param.value * 10000) / 100) + '%';
                }"""
        ),
        position="inside",
    ),
)

c2 = Grid().add(l1, grid_opts=opts.GridOpts()).add(l2, grid_opts=opts.GridOpts())

# ---------------------------------------------- Liquid - Liquid_shape_rect ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid
from pyecharts.globals import SymbolType

c3 = (
    Liquid()
    .add("lq", [0.3, 0.7], is_outline_show=False, shape=SymbolType.RECT)
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-Shape-rect"))
)

# ---------------------------------------------- Liquid - Liquid_base ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid

c4 = (
    Liquid()
    .add("lq", [0.6, 0.7])
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-基本示例"))
)

# ---------------------------------------------- Liquid - Liquid_data_precision ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid
from pyecharts.commons.utils import JsCode

c5 = (
    Liquid()
    .add(
        "lq",
        [0.3254],
        label_opts=opts.LabelOpts(
            font_size=50,
            formatter=JsCode(
                """function (param) {
                    return (Math.floor(param.value * 10000) / 100) + '%';
                }"""
            ),
            position="inside",
        ),
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-数据精度"))
)

# ---------------------------------------------- Liquid - Liquid_without_outline ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid

c6 = (
    Liquid()
    .add("lq", [0.6, 0.7, 0.8], is_outline_show=False)
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-无边框"))
)

# ---------------------------------------------- Liquid - Liquid_shape_arrow ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Liquid
from pyecharts.globals import SymbolType

c7 = (
    Liquid()
    .add("lq", [0.3, 0.7], is_outline_show=False, shape=SymbolType.ARROW)
    .set_global_opts(title_opts=opts.TitleOpts(title="Liquid-Shape-arrow"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="liquid",
    public=True,
)

swanlab.log(
    {
        "Liquid - Liquid_shape_diamond": c1,
        "Liquid - Multiple_liquid": c2,
        "Liquid - Liquid_shape_rect": c3,
        "Liquid - Liquid_base": c4,
        "Liquid - Liquid_data_precision": c5,
        "Liquid - Liquid_without_outline": c6,
        "Liquid - Liquid_shape_arrow": c7
    }
)



from swanlab.integration.keras import SwanLabLogger
import tensorflow as tf
import swanlab

# Initialize SwanLab
swanlab.init(
    project="keras_mnist",
    experiment_name="mnist_example",
    description="Keras MNIST Example"
    )

# Load and preprocess MNIST data
(x_train, y_train), (x_test, y_test) = tf.keras.datasets.mnist.load_data()
x_train = x_train.reshape(-1, 28, 28, 1).astype('float32') / 255.0
x_test = x_test.reshape(-1, 28, 28, 1).astype('float32') / 255.0

# Build a simple CNN model
model = tf.keras.Sequential([
    tf.keras.layers.Conv2D(32, 3, activation='relu', input_shape=(28, 28, 1)),
    tf.keras.layers.MaxPooling2D(),
    tf.keras.layers.Conv2D(64, 3, activation='relu'),
    tf.keras.layers.MaxPooling2D(),
    tf.keras.layers.Flatten(),
    tf.keras.layers.Dense(64, activation='relu'),
    tf.keras.layers.Dense(10, activation='softmax')
])

# Compile the model
model.compile(
    optimizer='adam',
    loss='sparse_categorical_crossentropy',
    metrics=['accuracy']
)

# Train the model with SwanLabLogger
model.fit(
    x_train, 
    y_train,
    epochs=5,
    validation_data=(x_test, y_test),
    callbacks=[SwanLabLogger()]
)



"""
@author: cunyue
@file: __init__.py
@time: 2025/6/19 16:05
@description: 全局、用户可访问工具依赖
"""

from swankit.callback import SwanKitCallback
from swankit.callback.models import *
from swankit.core import *
from swankit.env import *

from .logger import SwanKitLogger

__all__ = [
    # logger
    "SwanKitLogger",
    # env
    "SwanLabSharedEnv",
    "SwanLabMode",
    "create_time",
    "is_macos",
    "is_windows",
    "get_save_dir",
    "get_swanlog_dir",
    "get_mode",
    # callback
    "SwanKitCallback",
    "ColumnInfo",
    "MetricInfo",
    "MetricErrorInfo",
    "RuntimeInfo",
    "ColumnClass",
    "SectionType",
    "ColumnConfig",
    "YRange",
    # core
    "BaseType",
    "MediaType",
    "ChartType",
    "DataSuite",
    "MediaBuffer",
    "ParseResult",
    "ParseErrorInfo",
    "ChartReference",
]



"""
// @author: ComPleHN
// @file: grid.vue
// @time: 2025/5/27 16:39
// @description: 本文件是对于echarts的 组合组件 图表的测试
"""
# ---------------------------------------------- Grid - Grid_overlap_multi_xy_axis ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line

bar = (
    Bar()
    .add_xaxis(["{}月".format(i) for i in range(1, 13)])
    .add_yaxis(
        "蒸发量",
        [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
        yaxis_index=0,
        color="#d14a61",
    )
    .add_yaxis(
        "降水量",
        [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
        yaxis_index=1,
        color="#5793f3",
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            name="蒸发量",
            type_="value",
            min_=0,
            max_=250,
            position="right",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#d14a61")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        )
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            type_="value",
            name="温度",
            min_=0,
            max_=25,
            position="left",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#675bba")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
            splitline_opts=opts.SplitLineOpts(
                is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
            ),
        )
    )
    .set_global_opts(
        yaxis_opts=opts.AxisOpts(
            name="降水量",
            min_=0,
            max_=250,
            position="right",
            offset=80,
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#5793f3")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        ),
        title_opts=opts.TitleOpts(title="Grid-Overlap-多 X/Y 轴示例"),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        legend_opts=opts.LegendOpts(pos_left="25%"),
    )
)

line = (
    Line()
    .add_xaxis(["{}月".format(i) for i in range(1, 13)])
    .add_yaxis(
        "平均温度",
        [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
        yaxis_index=2,
        color="#675bba",
        label_opts=opts.LabelOpts(is_show=False),
    )
)

bar1 = (
    Bar()
    .add_xaxis(["{}月".format(i) for i in range(1, 13)])
    .add_yaxis(
        "蒸发量 1",
        [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
        color="#d14a61",
        xaxis_index=1,
        yaxis_index=3,
    )
    .add_yaxis(
        "降水量 2",
        [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
        color="#5793f3",
        xaxis_index=1,
        yaxis_index=4,
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            name="蒸发量",
            type_="value",
            min_=0,
            max_=250,
            position="right",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#d14a61")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        )
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            type_="value",
            name="温度",
            min_=0,
            max_=25,
            position="left",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#675bba")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
            splitline_opts=opts.SplitLineOpts(
                is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
            ),
        )
    )
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(grid_index=1),
        yaxis_opts=opts.AxisOpts(
            name="降水量",
            min_=0,
            max_=250,
            position="right",
            offset=80,
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#5793f3")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        ),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
        legend_opts=opts.LegendOpts(pos_left="65%"),
    )
)

line1 = (
    Line()
    .add_xaxis(["{}月".format(i) for i in range(1, 13)])
    .add_yaxis(
        "平均温度 1",
        [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
        color="#675bba",
        label_opts=opts.LabelOpts(is_show=False),
        xaxis_index=1,
        yaxis_index=5,
    )
)

overlap_1 = bar.overlap(line)
overlap_2 = bar1.overlap(line1)

c1 = (
    Grid(init_opts=opts.InitOpts(width="1200px", height="800px"))
    .add(
        overlap_1, grid_opts=opts.GridOpts(pos_right="58%"), is_control_axis_index=True
    )
    .add(overlap_2, grid_opts=opts.GridOpts(pos_left="58%"), is_control_axis_index=True)
)

# ---------------------------------------------- Grid - Grid_multi_yaxis ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line

x_data = ["{}月".format(i) for i in range(1, 13)]
bar = (
    Bar()
    .add_xaxis(x_data)
    .add_yaxis(
        "蒸发量",
        [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3],
        yaxis_index=0,
        color="#d14a61",
    )
    .add_yaxis(
        "降水量",
        [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3],
        yaxis_index=1,
        color="#5793f3",
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            name="蒸发量",
            type_="value",
            min_=0,
            max_=250,
            position="right",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#d14a61")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        )
    )
    .extend_axis(
        yaxis=opts.AxisOpts(
            type_="value",
            name="温度",
            min_=0,
            max_=25,
            position="left",
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#675bba")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} °C"),
            splitline_opts=opts.SplitLineOpts(
                is_show=True, linestyle_opts=opts.LineStyleOpts(opacity=1)
            ),
        )
    )
    .set_global_opts(
        yaxis_opts=opts.AxisOpts(
            name="降水量",
            min_=0,
            max_=250,
            position="right",
            offset=80,
            axisline_opts=opts.AxisLineOpts(
                linestyle_opts=opts.LineStyleOpts(color="#5793f3")
            ),
            axislabel_opts=opts.LabelOpts(formatter="{value} ml"),
        ),
        title_opts=opts.TitleOpts(title="Grid-多 Y 轴示例"),
        tooltip_opts=opts.TooltipOpts(trigger="axis", axis_pointer_type="cross"),
    )
)

line = (
    Line()
    .add_xaxis(x_data)
    .add_yaxis(
        "平均温度",
        [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2],
        yaxis_index=2,
        color="#675bba",
        label_opts=opts.LabelOpts(is_show=False),
    )
)

bar.overlap(line)
c2 = Grid()
c2.add(bar, opts.GridOpts(pos_left="5%", pos_right="20%"), is_control_axis_index=True)

# ---------------------------------------------- Grid - Grid_vertical ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Grid, Line
from pyecharts.faker import Faker

bar = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Grid-Bar"))
)
line = (
    Line()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Grid-Line", pos_top="48%"),
        legend_opts=opts.LegendOpts(pos_top="48%"),
    )
)

c3 = (
    Grid()
    .add(bar, grid_opts=opts.GridOpts(pos_bottom="60%"))
    .add(line, grid_opts=opts.GridOpts(pos_top="60%"))
)

# ---------------------------------------------- Grid - Grid_geo_bar ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Geo, Grid
from pyecharts.faker import Faker

bar = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(legend_opts=opts.LegendOpts(pos_left="20%"))
)
geo = (
    Geo()
    .add_schema(maptype="china")
    .add("geo", [list(z) for z in zip(Faker.provinces, Faker.values())])
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        visualmap_opts=opts.VisualMapOpts(),
        title_opts=opts.TitleOpts(title="Grid-Geo-Bar"),
    )
)

c4 = (
    Grid()
    .add(bar, grid_opts=opts.GridOpts(pos_top="50%", pos_right="75%"))
    .add(geo, grid_opts=opts.GridOpts(pos_left="60%"))
)

# ---------------------------------------------- Grid - Grid_horizontal ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Grid, Line, Scatter
from pyecharts.faker import Faker

scatter = (
    Scatter()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Grid-Scatter"),
        legend_opts=opts.LegendOpts(pos_left="20%"),
    )
)
line = (
    Line()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Grid-Line", pos_right="5%"),
        legend_opts=opts.LegendOpts(pos_right="20%"),
    )
)

c5 = (
    Grid()
    .add(scatter, grid_opts=opts.GridOpts(pos_left="55%"))
    .add(line, grid_opts=opts.GridOpts(pos_right="55%"))
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="grid",
    public=True,
)

swanlab.log(
    {
        "Grid - Grid_overlap_multi_xy_axis": c1,
        "Grid - Grid_multi_yaxis": c2,
        "Grid - Grid_vertical": c3,
        "Grid - Grid_geo_bar": c4,
        "Grid - Grid_horizontal": c5
    }
)



"""
// @author: ComPleHN
// @file: bar2.vue
// @time: 2025/5/27 12:45
// @description: 由于bar类图表过多,我将分为多个文件进行测试
"""
# ---------------------------------------------- Bar - Bar_markline_type ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c1 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-MarkLine（指定类型）"))
    .set_series_opts(
        label_opts=opts.LabelOpts(is_show=False),
        markline_opts=opts.MarkLineOpts(
            data=[
                opts.MarkLineItem(type_="min", name="最小值"),
                opts.MarkLineItem(type_="max", name="最大值"),
                opts.MarkLineItem(type_="average", name="平均值"),
            ]
        ),
    )
)

# ---------------------------------------------- Bar - Bar_graphic_component ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker

c2 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-Graphic 组件示例"),
        graphic_opts=[
            opts.GraphicGroup(
                graphic_item=opts.GraphicItem(
                    rotation=JsCode("Math.PI / 4"),
                    bounding="raw",
                    right=110,
                    bottom=110,
                    z=100,
                ),
                children=[
                    opts.GraphicRect(
                        graphic_item=opts.GraphicItem(
                            left="center", top="center", z=100
                        ),
                        graphic_shape_opts=opts.GraphicShapeOpts(width=400, height=50),
                        graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                            fill="rgba(0,0,0,0.3)"
                        ),
                    ),
                    opts.GraphicText(
                        graphic_item=opts.GraphicItem(
                            left="center", top="center", z=100
                        ),
                        graphic_textstyle_opts=opts.GraphicTextStyleOpts(
                            text="pyecharts bar chart",
                            font="bold 26px Microsoft YaHei",
                            graphic_basicstyle_opts=opts.GraphicBasicStyleOpts(
                                fill="#fff"
                            ),
                        ),
                    ),
                ],
            )
        ],
    )
)

# ---------------------------------------------- Bar - Bar_waterfall_plot ----------------------------------------------
from pyecharts.charts import Bar
from pyecharts import options as opts

x_data = [f"11月{str(i)}日" for i in range(1, 12)]
y_total = [0, 900, 1245, 1530, 1376, 1376, 1511, 1689, 1856, 1495, 1292]
y_in = [900, 345, 393, "-", "-", 135, 178, 286, "-", "-", "-"]
y_out = ["-", "-", "-", 108, 154, "-", "-", "-", 119, 361, 203]


c3 = (
    Bar()
    .add_xaxis(xaxis_data=x_data)
    .add_yaxis(
        series_name="",
        y_axis=y_total,
        stack="总量",
        itemstyle_opts=opts.ItemStyleOpts(color="rgba(0,0,0,0)"),
    )
    .add_yaxis(series_name="收入", y_axis=y_in, stack="总量")
    .add_yaxis(series_name="支出", y_axis=y_out, stack="总量")
    .set_global_opts(yaxis_opts=opts.AxisOpts(type_="value"))
)

# ---------------------------------------------- Bar - Bar_border_radius ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.commons.utils import JsCode
from pyecharts.faker import Faker

c4 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values(), category_gap="60%")
    .set_series_opts(
        itemstyle_opts={
            "normal": {
                "color": JsCode(
                    """new echarts.graphic.LinearGradient(0, 0, 0, 1, [{
                offset: 0,
                color: 'rgba(0, 244, 255, 1)'
            }, {
                offset: 1,
                color: 'rgba(0, 77, 167, 1)'
            }], false)"""
                ),
                "barBorderRadius": [30, 30, 30, 30],
                "shadowColor": "rgb(0, 160, 221)",
            }
        }
    )
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-渐变圆柱"))
)

# ---------------------------------------------- Bar - Bar_same_series_gap ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker


c5 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values(), category_gap="80%")
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-单系列柱间距离"))
)

# ---------------------------------------------- Bar - Bar_datazoom_inside ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c6 = (
    Bar()
    .add_xaxis(Faker.days_attrs)
    .add_yaxis("商家A", Faker.days_values, color=Faker.rand_color())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-DataZoom（inside）"),
        datazoom_opts=opts.DataZoomOpts(type_="inside"),
    )
)

# ---------------------------------------------- Bar - Bar_is_selected ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker


c7 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-默认取消显示某 Series"),
        legend_opts=opts.LegendOpts(selected_map={"商家B": False}),
    )
)

# ---------------------------------------------- Bar - Bar_reversal_axis ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c8 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .reversal_axis()
    .set_series_opts(label_opts=opts.LabelOpts(position="right"))
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-翻转 XY 轴"))
)

# ---------------------------------------------- Bar - Bar_markpoint_custom ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

x, y = Faker.choose(), Faker.values()
c9 = (
    Bar()
    .add_xaxis(x)
    .add_yaxis(
        "商家A",
        y,
        markpoint_opts=opts.MarkPointOpts(
            data=[opts.MarkPointItem(name="自定义标记点", coord=[x[2], y[2]], value=y[2])]
        ),
    )
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-MarkPoint（自定义）"))
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
)

# ---------------------------------------------- Bar - Bar_base_with_animation ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker


c10 = (
    Bar(
        init_opts=opts.InitOpts(
            animation_opts=opts.AnimationOpts(
                animation_delay=1000, animation_easing="elasticOut"
            )
        )
    )
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-动画配置基本示例", subtitle="我是副标题"))
)

# ---------------------------------------------- Bar - Bar_histogram ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c11 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values(), category_gap=0, color=Faker.rand_color())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-直方图"))
)

# ---------------------------------------------- Bar - Bar_markline_custom ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c12 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-MarkLine（自定义）"))
    .set_series_opts(
        label_opts=opts.LabelOpts(is_show=False),
        markline_opts=opts.MarkLineOpts(
            data=[opts.MarkLineItem(y=50, name="yAxis=50")]
        ),
    )
)

# ---------------------------------------------- Bar - Bar_base ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker


c13 = (
    Bar()
    .add_xaxis(Faker.choose())
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Bar-基本示例", subtitle="我是副标题"))
)

# ---------------------------------------------- Bar - Bar_datazoom_both ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar
from pyecharts.faker import Faker

c14 = (
    Bar()
    .add_xaxis(Faker.days_attrs)
    .add_yaxis("商家A", Faker.days_values, color=Faker.rand_color())
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Bar-DataZoom（slider+inside）"),
        datazoom_opts=[opts.DataZoomOpts(), opts.DataZoomOpts(type_="inside")],
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="bar2",
    public=True,
)

swanlab.log(
    {
        "Bar_markline_type": c1,
        "Bar_graphic_component": c2,
        "Bar_waterfall_plot": c3,
        "Bar_border_radius": c4,
        "Bar_same_series_gap": c5,
        "Bar_datazoom_inside": c6,
        "Bar_is_selected": c7,
        "Bar_reversal_axis": c8,
        "Bar_markpoint_custom": c9,
        "Bar_base_with_animation": c10,
        "Bar_histogram": c11,
        "Bar_markline_custom": c12,
        "Bar_base": c13,
        "Bar_datazoom_both": c14
        
    }
)


#!/usr/bin/env python# -*- coding: utf-8 -*-
r"""
@DATE: 2025-3-20
@File: test/unit/test_swanlab_settings.py
@IDE: cursor
@Description:
    SwanLab settings module unit tests
"""
import platform

import pytest
from pydantic import ValidationError

import swanlab
from swanlab.swanlab_settings import get_settings, reset_settings
from tutils import TEMP_PATH


class TestSwanlabSettingsBasics:
    def test_settings_frozen(self):
        """测试Settings对象实例化后是不可变的"""
        settings = swanlab.Settings()
        with pytest.raises(ValidationError):
            settings.hardware_monitor = False

    def test_settings_initialization(self):
        """测试Settings的初始化和验证"""
        # 测试基本初始化
        settings = swanlab.Settings()
        assert settings.hardware_monitor is True

        # 测试自定义参数
        settings = swanlab.Settings(hardware_monitor=False)
        assert settings.hardware_monitor is False


def test_log_proxy_type():
    """测试日志代理类型"""
    # 测试默认值
    settings = swanlab.Settings()
    assert settings.log_proxy_type == "all"

    # 测试自定义值
    settings = swanlab.Settings(log_proxy_type="stdout")
    assert settings.log_proxy_type == "stdout"

    # 测试无效值
    with pytest.raises(ValidationError):
        swanlab.Settings(log_proxy_type="invalid")


class TestSwanlabSettings:
    def teardown_method(self):
        # 每个测试方法后执行
        try:
            swanlab.finish()
        except Exception:
            pass
        reset_settings()

    def test_merge_settings(self):
        """测试合并设置到全局状态"""
        # 合并自定义设置
        settings1 = swanlab.Settings(hardware_monitor=False)
        swanlab.merge_settings(settings1)

        settings = get_settings()
        assert settings.hardware_monitor is False

        # 再次修改设置
        settings2 = swanlab.Settings(hardware_monitor=True)
        swanlab.merge_settings(settings2)

        settings = get_settings()
        assert settings.hardware_monitor is True

    def test_default_setup(self):
        """测试不提供设置时的默认行为"""
        # 不提供设置执行init
        swanlab.init(mode="disabled")

        # 验证使用了默认设置
        settings = get_settings()
        assert settings.hardware_monitor is True

    def test_change_settings(self):
        """测试修改设置"""
        settings = swanlab.Settings(hardware_monitor=False)
        swanlab.merge_settings(settings)

        settings = get_settings()
        assert settings.hardware_monitor is False

        new_settings = swanlab.Settings(hardware_monitor=True)
        swanlab.merge_settings(new_settings)

        settings = get_settings()
        assert settings.hardware_monitor is True

    def test_change_settings_with_init(self):
        """测试在init时修改设置"""
        settings = swanlab.Settings(hardware_monitor=False)
        swanlab.merge_settings(settings)
        settings = get_settings()  # 此时硬件监控被关闭
        assert settings.hardware_monitor is False

        new_settings = swanlab.Settings(hardware_monitor=True)
        swanlab.init(
            settings=new_settings,
            mode="disabled",
        )  # 此时硬件监控被开启
        settings = get_settings()
        assert settings.hardware_monitor is True

    def test_type_validation(self):
        """测试类型验证"""
        # 传入非Settings对象到merge_settings
        with pytest.raises(TypeError):
            swanlab.merge_settings({"hardware_monitor": False})

        # 传入非Settings对象到init
        with pytest.raises(TypeError):
            swanlab.init(settings={"hardware_monitor": False})

        # 传入基本类型到merge_settings
        with pytest.raises(TypeError):
            swanlab.merge_settings(42)

        with pytest.raises(TypeError):
            swanlab.merge_settings("settings")

        with pytest.raises(TypeError):
            swanlab.merge_settings(True)

        # 传入其他非Settings类的对象
        class FakeSettings:
            hardware_monitor = False

        with pytest.raises(TypeError):
            swanlab.merge_settings(FakeSettings())

        # 测试嵌套对象类型验证
        with pytest.raises(TypeError):
            swanlab.init(settings={"hardware_monitor": False, "nested": {"invalid": True}})

    def test_max_log_length(self):
        """
        测试日志长度范围
        """
        swanlab.Settings(max_log_length=500)
        swanlab.Settings(max_log_length=4096)
        with pytest.raises(ValidationError):
            swanlab.Settings(max_log_length=499)

        with pytest.raises(ValidationError):
            swanlab.Settings(max_log_length=4097)

    def test_disk_io_default_dir(self):
        """
        测试默认磁盘IO监控路径
        """
        settings = swanlab.Settings()
        assert settings.disk_io_dir == "C:\\" if platform.system() == 'Windows' else "/"

    def test_hardware_interval(self):
        settings = swanlab.Settings()
        assert settings.hardware_interval is None
        # 最小值为5
        with pytest.raises(ValidationError):
            swanlab.Settings(hardware_interval=1)
        # 不允许为小数
        with pytest.raises(ValidationError):
            swanlab.Settings(hardware_interval=5.5)


def test_filter_changed_fields():
    # 默认
    settings = swanlab.Settings()
    changed = settings.filter_changed_fields()
    assert changed == {}
    # 修改
    settings = swanlab.Settings(hardware_monitor=False)
    changed = settings.filter_changed_fields()
    assert changed == {"hardware_monitor": False}
    # 修改，但是没有变化
    settings = swanlab.Settings(hardware_monitor=True)
    changed = settings.filter_changed_fields()
    assert changed == {}

    # 修改一些特殊不可被JSON序列化的字段
    settings = swanlab.Settings(disk_io_dir=TEMP_PATH)
    changed = settings.filter_changed_fields()
    assert changed == {"disk_io_dir": TEMP_PATH}



import json
import netrc
import os
import time

import nanoid
import pytest

import swanlab.package as P
from swanlab.env import SwanLabEnv, get_save_dir

_ = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
package_data = json.load(open(os.path.join(_, "swanlab", "package.json")))


def test_package_latest_version():
    """
    测试获取swanlab最新版本号
    """
    # 正常情况
    assert isinstance(P.get_package_latest_version(), str)
    # 超时情况
    assert P.get_package_latest_version(timeout=1e-3) is None


def test_get_package_version():
    """
    测试获取版本号
    """
    assert P.get_package_version() == package_data["version"]


def test_get_host_web_env():
    """
    通过环境变量指定web地址
    """
    os.environ[SwanLabEnv.WEB_HOST.value] = nanoid.generate()
    assert P.get_host_web() == os.environ[SwanLabEnv.WEB_HOST.value]


def test_get_host_api_env():
    """
    通过环境变量指定api地址
    """
    os.environ[SwanLabEnv.API_HOST.value] = nanoid.generate()
    assert P.get_host_api() == os.environ[SwanLabEnv.API_HOST.value]


def test_fmt_web_host():
    """
    测试格式化web地址
    """
    assert P.fmt_web_host() == P.get_host_web().rstrip("/")
    assert P.fmt_web_host("https://abc.cn/") == "https://abc.cn"


# ---------------------------------- 登录部分 ----------------------------------


class TestGetKey:
    @staticmethod
    def remove_env_key():
        if SwanLabEnv.API_KEY.value in os.environ:
            del os.environ[SwanLabEnv.API_KEY.value]

    def test_ok(self):
        """
        获取key成功
        """
        self.remove_env_key()
        # 首先需要登录
        file = os.path.join(get_save_dir(), ".netrc")
        with open(file, "w"):
            pass
        nrc = netrc.netrc(file)
        key = nanoid.generate()
        nrc.hosts[P.get_host_api()] = ("user", "", key)
        with open(file, "w") as f:
            f.write(nrc.__repr__())
        assert P.get_key() == key

    def test_no_file(self):
        """
        文件不存在
        """
        self.remove_env_key()
        from swanlab.error import KeyFileError

        with pytest.raises(KeyFileError) as e:
            P.get_key()
        assert str(e.value) == "The file does not exist"

    def test_no_host(self):
        from swanlab.error import KeyFileError

        self.test_ok()  # 此时删除了环境变量
        host = nanoid.generate()
        os.environ[SwanLabEnv.API_HOST.value] = host
        assert P.get_host_api() == host
        with pytest.raises(KeyFileError) as e:
            P.get_key()
        time.sleep(0.5)
        assert str(e.value) == f"The host {host} does not exist"

    def test_use_env(self):
        """
        使用环境变量，优先级高于本地文件
        """
        self.test_ok()
        key = nanoid.generate()
        os.environ[SwanLabEnv.API_KEY.value] = key
        assert P.get_key() == key

    def test_compatible(self):
        """
        测试向下兼容性，假设有一个带 /api 的host在本地文件中，但是寻找的是不带 /api 的host
        """
        self.remove_env_key()
        file = os.path.join(get_save_dir(), ".netrc")
        with open(file, "w"):
            pass
        nrc = netrc.netrc(file)
        key = nanoid.generate()
        host = "https://api.swanlab.cn/api"
        os.environ[SwanLabEnv.API_HOST.value] = host
        nrc.hosts[host] = ("user", "", key)
        with open(file, "w") as f:
            f.write(repr(nrc))
        assert P.get_key() == key
        host = host.rstrip("/api")
        os.environ[SwanLabEnv.API_HOST.value] = host
        self.remove_env_key()
        assert P.get_key() == key


class TestSaveKey:

    @staticmethod
    def get_key(path, host):
        nrc = netrc.netrc(path)
        info = nrc.authenticators(host)
        if info is None:
            info = nrc.authenticators(host.rstrip("/api"))
        return info[2]

    @staticmethod
    def loose_compare(a, b, equal: bool = True):
        """
        不同文件系统的时间精度不同，因此需要 sleep 一段时间进行比较
        不精确到微妙 只精确到毫秒
        """
        assert (abs(a - b) < 1e-5) is equal

    def test_ok(self):
        """
        保存key成功
        """
        path = os.path.join(get_save_dir(), ".netrc")
        password = nanoid.generate()
        host = P.get_host_api()
        P.save_key("user", password, host=host)
        assert self.get_key(path, host) == password
        # 在保存一次，保证只存在一个host
        new_host = nanoid.generate()
        P.save_key("user", password, host=new_host)
        nrc = netrc.netrc(path)
        assert len(nrc.hosts) == 1
        assert nrc.authenticators(new_host) is not None

    def test_host(self):
        """
        测试 host 的正确性
        """
        path = os.path.join(get_save_dir(), ".netrc")
        password = nanoid.generate()
        host = "https://swanlab.ai/api"
        P.save_key("user", password, host=host)
        nrc = netrc.netrc(path)
        assert len(nrc.hosts) == 1
        assert nrc.authenticators("https://swanlab.ai") is not None

    def test_duplicate(self):
        """
        测试重复保存，此时会略过保存，因此不会改变文件的修改时间
        """

        path = os.path.join(get_save_dir(), ".netrc")

        def duplicate_save(p: str, h: str, user: str = "user", equal: bool = True):
            c = os.path.getmtime(path)
            time.sleep(0.1)
            P.save_key(user, p, host=h)
            assert self.get_key(path, h) == p
            return self.loose_compare(os.path.getmtime(path), c, equal)

        password = "123456"
        host = P.get_host_api()
        P.save_key("user", password, host=host)
        assert self.get_key(path, host) == password

        # 重复保存
        duplicate_save(password, host, equal=True)
        # 再次保存，但是账号不同
        new_password = "567890"
        duplicate_save(new_password, host, user="user2", equal=False)
        # 再次保存，但是host不同
        new_host = "example.com"
        duplicate_save(new_password, new_host, equal=False)
        nrc = netrc.netrc(path)
        assert len(nrc.hosts) == 1
        assert list(nrc.hosts.keys())[0] == new_host
        assert nrc.authenticators(new_host) is not None
        # 再次保存，但是密码又不同了
        new_password = "abcdef"
        duplicate_save(new_password, new_host, equal=False)
        nrc = netrc.netrc(path)
        assert len(nrc.hosts) == 1
        assert nrc.authenticators(new_host)[2] == new_password
        # 再次保存，但是密码又相同了
        duplicate_save(new_password, new_host, equal=True)


class TestHasApiKey:
    @staticmethod
    def save_api_key():
        path = os.path.join(get_save_dir(), ".netrc")
        with open(path, "w"):
            pass
        nrc = netrc.netrc(path)
        key = nanoid.generate()
        nrc.hosts[P.get_host_api()] = ("user", "", key)
        with open(path, "w") as f:
            f.write(nrc.__repr__())

    def test_ok(self):
        """
        已经登录
        """
        self.save_api_key()
        assert P.has_api_key()

    def test_no_file(self):
        """
        文件不存在
        """
        assert not P.has_api_key()

    def test_wrong_host(self):
        """
        host不匹配
        """
        self.save_api_key()
        os.environ[SwanLabEnv.API_HOST.value] = nanoid.generate()
        assert not P.has_api_key()


class TestHostFormatter:
    def test_ok(self):
        formatter = P.HostFormatter()
        assert formatter.fmt("swanlab.cn") == "https://swanlab.cn"
        assert formatter.fmt("https://swanlab.cn") == "https://swanlab.cn"
        assert formatter.fmt("http://swanlab.cn") == "http://swanlab.cn"  # noqa
        assert formatter.fmt("https://swanlab.cn:8443/") == "https://swanlab.cn:8443"
        assert formatter.fmt("abc.example.com") == "https://abc.example.com"
        assert formatter.fmt("localhost") == "https://localhost"
        assert formatter.fmt("http://localhost") == "http://localhost"
        assert formatter.fmt("http://localhost:8080") == "http://localhost:8080"
        assert formatter.fmt("http://127.0.0.1:8080/") == "http://127.0.0.1:8080"

    def test_value_err(self):
        formatter = P.HostFormatter()
        with pytest.raises(ValueError):
            formatter.fmt("test")
        with pytest.raises(ValueError):
            formatter.fmt("https://test")
        with pytest.raises(ValueError):
            formatter.fmt("http://test")  # noqa

    def test_env_var(self):
        """
        输入正确的host，会自动赋值给空web_host
        """
        host = "https://swanlab.cn"
        P.HostFormatter(host=host)()
        assert os.environ[SwanLabEnv.WEB_HOST.value] == host
        assert os.environ[SwanLabEnv.API_HOST.value] == host + "/api"



"""
@author: cunyue
@file: test_apple.py
@time: 2024/12/10 20:33
@description: 测试苹果芯片信息采集
"""

import pytest

from swanlab.data.run.metadata.hardware.soc.apple import AppleChipCollector
from swanlab.toolkit import is_macos


@pytest.mark.skipif(not is_macos(), reason="Apple chip info only available on macOS")
def test_apple_collector():
    apple = AppleChipCollector()
    apple()
    assert apple.collect_num == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
@DATE: 2024/4/26 16:40
@File: config.py
@IDE: pycharm
@Description:
    SwanLabConfig 配置类
"""
import argparse
import datetime
import json
import math
import re
from collections.abc import MutableMapping
from dataclasses import is_dataclass, asdict
from typing import Any, Union
from typing import Callable, Optional

import yaml

from swanlab.data.modules import Line
from swanlab.log import swanlog
from swanlab.toolkit import RuntimeInfo

BASE_TYPE = (int, float, str)


def json_serializable(obj):
    """
    将传入的字典转换为JSON可序列化格式。
    :raises TypeError: 对象不是JSON可序列化的
    """
    # 如果对象是基本类型，则直接返回
    # 不可以直接使用isinstance，详见issue: https://github.com/SwanHubX/SwanLab/issues/702
    if obj is None:
        return None
    if type(obj) is float and math.isnan(obj):
        return Line.nan
    if type(obj) is float and math.isinf(obj):
        return Line.inf

    # bool 类型需要特殊处理，因为bool继承自int: https://github.com/SwanHubX/SwanLab/issues/1035
    if type(obj) is bool:
        return obj

    for t in BASE_TYPE:
        if type(obj) is t:
            return obj
        # 继承的子类需要转译
        if isinstance(obj, t):
            return t(obj)

    # 将日期和时间转换为字符串
    if isinstance(obj, (datetime.date, datetime.datetime)):
        return obj.isoformat()

    # 对于列表和元组，递归调用此函数
    elif isinstance(obj, (list, tuple)):
        return [json_serializable(item) for item in obj]

    # 对于字典，递归调用此函数处理值，并将key转换为字典
    elif isinstance(obj, dict):
        return {str(key): json_serializable(value) for key, value in obj.items()}

    # 对于可变映射，递归调用此函数处理值，并将key转换为字典
    elif isinstance(obj, MutableMapping):
        return {str(key): json_serializable(value) for key, value in obj.items()}

    try:
        return str(obj)
    except Exception:
        raise TypeError(f"Object: {obj} is not JSON serializable")


def third_party_config_process(data) -> dict:
    """
    对于一些特殊的第三方库的处理，例如omegaconf
    :raises TypeError: 适配的写入的第三方库都没有命中，抛出TypeError
    """
    # 如果是omegaconf的DictConfig，则转换为字典
    try:
        import omegaconf  # noqa

        if isinstance(data, omegaconf.DictConfig):
            return omegaconf.OmegaConf.to_container(data, resolve=True, throw_on_missing=True)
    except ImportError:
        pass

    # 如果是mmengine的Config，则转换为字典
    try:
        import mmengine  # noqa

        if isinstance(data, mmengine.Config):
            return mmengine.Config.to_dict(data)
    except ImportError:
        pass

    # 如果是argparse的Namespace，则转换为字典
    if isinstance(data, argparse.Namespace):
        return vars(data)

    # 如果是dataclass类，转换为字典
    if is_dataclass(data):
        return asdict(data)

    raise TypeError


def parse(config) -> dict:
    """
    Check the configuration item and convert it to a JSON serializable format.
    """
    if config is None:
        return {}

    # 1. 第三方配置类型判断与转换
    try:
        return third_party_config_process(config)
    except TypeError:
        pass

    # 2. 将config转换为可被json序列化的字典
    try:
        return json_serializable(config)
    except TypeError:  # noqa
        pass

    # 3. 尝试序列化，序列化成功直接返回
    try:
        return json.loads(json.dumps(config))
    except Exception as e:  # noqa
        # 还失败就没办法了，👋
        raise TypeError(f"config: {config} is not a json serialized dict, error: {e}")


__config_attr__ = ["_SwanLabConfig__config", "_SwanLabConfig__on_setter"]


class SwanLabConfig(MutableMapping):
    """
    The SwanConfig class is used for realize the invocation method of `run.config.lr`.

    Attention:
    The configuration item must be JSON serializable; Cannot set private attributes by `.__xxx`.
    """

    def __init__(
        self,
        config: Union[MutableMapping, argparse.Namespace] = None,
        on_setter: Optional[Callable[[RuntimeInfo], Any]] = None,
    ):
        """
        实例化配置类，如果settings不为None，说明是通过swanlab.init调用的，否则是通过swanlab.config调用的
        """
        # 每一个实例有自己的config
        self.__config = {}
        if config is not None:
            self.__config.update(parse(config))
        self.__on_setter = on_setter

    @staticmethod
    def fmt_config(config: dict):
        """
        格式化config，值改为value字段，增加desc和sort字段
        """
        # 遍历每一个配置项，值改为value
        sort = 0
        for key, value in config.items():
            config[key] = {"value": value, "desc": "", "sort": sort}
            sort += 1

    @staticmethod
    def revert_config(config: dict) -> dict:
        """
        重置格式化后的 config，删除 desc 和 sort 字段，但是保持排序
        """
        result_items = []
        result = {}
        for key, value in config.items():
            if isinstance(value, dict) and "value" in value:
                # 如果 sort 不存在默认排最前面
                result_items.append((key, value["value"], value.get("sort", 0)))
        # Sort by the "sort" field and reconstruct the dictionary
        for key, value, _ in sorted(result_items, key=lambda x: x[2]):
            result[key] = value
        return result

    def __save(self):
        """
        保存config为dict
        """
        if not self.__on_setter:
            return swanlog.debug("The configuration is not saved because the setter is not set.")
        try:
            # 深度拷贝一次，防止引用传递
            data = yaml.load(yaml.dump(self.__config), Loader=yaml.FullLoader)
        except Exception as e:
            swanlog.error(f"Error occurred when saving config: {e}")
            return
        # 遍历每一个配置项，值改为value，如果是字典，则递归调用
        self.fmt_config(data)
        r = RuntimeInfo(config=self.__config)
        self.__on_setter(r)
        swanlog.debug(f"Save configuration.")

    # ---------------------------------- 实现对象风格 ----------------------------------

    def __delattr__(self, name: str):
        """
        删除配置项，如果配置项不存在
        """
        # _*__正则开头的属性不允许删除
        if re.match(r"_.*__", name):
            raise AttributeError(f"Attribute '{name}' is private and cannot be deleted")
        try:
            del self.__config[name]
            self.__save()
        except KeyError:
            raise AttributeError(f"You have not deleted '{name}' in the config of the current experiment")

    def __getattr__(self, name: str):
        """
        如果以点号方式访问属性且属性不存在于类中，尝试从配置字典中获取。
        """
        try:
            return self.__config[name]
        except KeyError:
            raise AttributeError(f"You have not get '{name}' in the config of the current experiment")

    def __setattr__(self, name: str, value: Any) -> None:
        """
        Custom setter attribute, user can not set private attributes.
        """
        name = str(name)
        if name in __config_attr__:
            return super().__setattr__(name, value)
        # _*__正则开头的属性不允许设置
        if re.match(r"_.*__", name):
            raise AttributeError(f"Attribute '{name}' is private and cannot be set")
        # 否则应该设置到配置字典中
        self.__config[name] = parse(value)
        self.__save()

    # ---------------------------------- 实现字典风格 ----------------------------------

    def get(self, name: str, default=None):
        """
        Get the value of a configuration item. If the item does not exist, raise AttributeError.
        """
        try:
            return self.__config[name]
        except KeyError:
            return default

    def __delitem__(self, name: str):
        """
        删除配置项，如果配置项不存在,跳过
        """
        try:
            del self.__config[name]
            self.__save()
        except KeyError:
            raise KeyError(f"You have not set '{name}' in the config of the current experiment when deleting")

    def __getitem__(self, name: str):
        """
        以字典方式获取配置项的值
        """
        # 如果self.__dict__中有name属性，则返回
        if not isinstance(name, str):
            raise TypeError(f"Key must be a string, but got {type(name)}")
        # 以_SwanLabConfig__开头，删除
        if name.startswith("_SwanLabConfig__"):
            name = name[15:]
        try:
            return self.__config[name]
        except KeyError:
            raise KeyError(f"You have not get '{name}' in the config of the current experiment")

    def __setitem__(self, name: str, value: Any) -> None:
        """
        Set the value of a configuration item. If the item does not exist, create it.
        User are not allowed to set private attributes.
        """
        name = str(name)
        self.__config[name] = parse(value)
        self.__save()

    def __iter__(self):
        """
        返回配置字典的迭代器。
        """
        return iter(self.__config)

    def __len__(self):
        """
        返回配置项的数量。
        """
        return len(self.__config)

    def __str__(self):
        return str(self.__config)

    # ---------------------------------- 其他函数 ----------------------------------

    def set(self, name: str, value: Any):
        """
        Explicitly set the value of a configuration item and save it.
        Private attributes are not allowed to be set.
        """
        name = str(name)
        self.__config[name] = parse(value)
        self.__save()

    def pop(self, name: str):
        """
        Delete a configuration item; if the item does not exist, skip.
        """
        try:
            t = self.__config[name]
            del self.__config[name]
            self.__save()
            return t
        except KeyError:
            return None

    def update(self, __m: Union[MutableMapping, argparse.Namespace] = None, **kwargs):
        """
        Update the configuration with the key/value pairs from __m, overwriting existing keys.
        """
        if __m is not None:
            for k, v in parse(__m).items():
                self.__config[k] = v
        for k, v in kwargs.items():
            self.__config[k] = parse(v)
        self.__save()

    def clean(self):
        """
        Clean the configuration.
        Attention: This method will reset the instance and instance will not automatically save the configuration.
        """
        self.__config.clear()
        self.__on_setter = None



"""
// @author: ComPleHN
// @file: dataset.vue
// @time: 2025/5/27 15:03
// @description: 本文件是对于echarts的 数据集 图表的测试,本文件有注意事项请看本目录下 README.md
"""

# ---------------------------------------------- Dataset - Dataset_pie ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Pie

c1 = (
    Pie()
    .add_dataset(
        source=[
            ["product", "2012", "2013", "2014", "2015", "2016", "2017"],
            ["Matcha Latte", 41.1, 30.4, 65.1, 53.3, 83.8, 98.7],
            ["Milk Tea", 86.5, 92.1, 85.7, 83.1, 73.4, 55.1],
            ["Cheese Cocoa", 24.1, 67.2, 79.5, 86.4, 65.2, 82.5],
            ["Walnut Brownie", 55.2, 67.1, 69.2, 72.4, 53.9, 39.1],
        ]
    )
    .add(
        series_name="Matcha Latte",
        data_pair=[],
        radius=60,
        center=["25%", "30%"],
        encode={"itemName": "product", "value": "2012"},
    )
    .add(
        series_name="Milk Tea",
        data_pair=[],
        radius=60,
        center=["75%", "30%"],
        encode={"itemName": "product", "value": "2013"},
    )
    .add(
        series_name="Cheese Cocoa",
        data_pair=[],
        radius=60,
        center=["25%", "75%"],
        encode={"itemName": "product", "value": "2014"},
    )
    .add(
        series_name="Walnut Brownie",
        data_pair=[],
        radius=60,
        center=["75%", "75%"],
        encode={"itemName": "product", "value": "2015"},
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Dataset simple pie example"),
        legend_opts=opts.LegendOpts(pos_left="30%", pos_top="2%"),
    )
)

# ---------------------------------------------- Dataset - Dataset_bar_0 ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar

c2 = (
    Bar()
    .add_dataset(
        source=[
            ["product", "2015", "2016", "2017"],
            ["Matcha Latte", 43.3, 85.8, 93.7],
            ["Milk Tea", 83.1, 73.4, 55.1],
            ["Cheese Cocoa", 86.4, 65.2, 82.5],
            ["Walnut Brownie", 72.4, 53.9, 39.1],
        ]
    )
    .add_yaxis(series_name="2015", y_axis=[])
    .add_yaxis(series_name="2016", y_axis=[])
    .add_yaxis(series_name="2017", y_axis=[])
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Dataset simple bar example"),
        xaxis_opts=opts.AxisOpts(type_="category"),
    )
)

# ---------------------------------------------- Dataset - Dataset_bar_1 ---------------------------------------------- 
from pyecharts import options as opts
from pyecharts.charts import Bar

c3 = (
    Bar()
    .add_dataset(
        source=[
            ["score", "amount", "product"],
            [89.3, 58212, "Matcha Latte"],
            [57.1, 78254, "Milk Tea"],
            [74.4, 41032, "Cheese Cocoa"],
            [50.1, 12755, "Cheese Brownie"],
            [89.7, 20145, "Matcha Cocoa"],
            [68.1, 79146, "Tea"],
            [19.6, 91852, "Orange Juice"],
            [10.6, 101852, "Lemon Juice"],
            [32.7, 20112, "Walnut Brownie"],
        ]
    )
    .add_yaxis(
        series_name="",
        y_axis=[],
        encode={"x": "amount", "y": "product"},
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Dataset normal bar example"),
        xaxis_opts=opts.AxisOpts(name="amount"),
        yaxis_opts=opts.AxisOpts(type_="category"),
        visualmap_opts=opts.VisualMapOpts(
            orient="horizontal",
            pos_left="center",
            min_=10,
            max_=100,
            range_text=["High Score", "Low Score"],
            dimension=0,
            range_color=["#D7DA8B", "#E15457"],
        ),
    )
)

# ---------------------------------------------- Dataset - Dataset_professional_scatter ----------------------------------------------
import json

from pyecharts import options as opts
from pyecharts.charts import Grid, Scatter

with open("../assets/echarts/dataset/life-expectancy-table.json", "r", encoding="utf-8") as f:
    j = json.load(f)

l1_1 = (
    Scatter()
    .add_dataset(
        dimensions=[
            "Income",
            "Life Expectancy",
            "Population",
            "Country",
            {"name": "Year", "type": "ordinal"},
        ],
        source=j,
    )
    .add_yaxis(
        series_name="",
        y_axis=[],
        symbol_size=2.5,
        xaxis_index=0,
        yaxis_index=0,
        encode={"x": "Income", "y": "Life Expectancy", "tooltip": [0, 1, 2, 3, 4]},
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(
            type_="value",
            grid_index=0,
            name="Income",
            axislabel_opts=opts.LabelOpts(rotate=50, interval=0),
        ),
        yaxis_opts=opts.AxisOpts(type_="value", grid_index=0, name="Life Expectancy"),
        title_opts=opts.TitleOpts(title="Encode and Matrix"),
    )
)

l1_2 = (
    Scatter()
    .add_dataset()
    .add_yaxis(
        series_name="",
        y_axis=[],
        symbol_size=2.5,
        xaxis_index=1,
        yaxis_index=1,
        encode={"x": "Country", "y": "Income", "tooltip": [0, 1, 2, 3, 4]},
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(
            type_="category",
            grid_index=1,
            name="Country",
            boundary_gap=False,
            axislabel_opts=opts.LabelOpts(rotate=50, interval=0),
        ),
        yaxis_opts=opts.AxisOpts(type_="value", grid_index=1, name="Income"),
    )
)

l2_1 = (
    Scatter()
    .add_dataset()
    .add_yaxis(
        series_name="",
        y_axis=[],
        symbol_size=2.5,
        xaxis_index=2,
        yaxis_index=2,
        encode={"x": "Income", "y": "Population", "tooltip": [0, 1, 2, 3, 4]},
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(
            type_="value",
            grid_index=2,
            name="Income",
            axislabel_opts=opts.LabelOpts(rotate=50, interval=0),
        ),
        yaxis_opts=opts.AxisOpts(type_="value", grid_index=2, name="Population"),
    )
)

l2_2 = (
    Scatter()
    .add_dataset()
    .add_yaxis(
        series_name="",
        y_axis=[],
        symbol_size=2.5,
        xaxis_index=3,
        yaxis_index=3,
        encode={"x": "Life Expectancy", "y": "Population", "tooltip": [0, 1, 2, 3, 4]},
        label_opts=opts.LabelOpts(is_show=False),
    )
    .set_global_opts(
        xaxis_opts=opts.AxisOpts(
            type_="value",
            grid_index=3,
            name="Life Expectancy",
            axislabel_opts=opts.LabelOpts(rotate=50, interval=0),
        ),
        yaxis_opts=opts.AxisOpts(type_="value", grid_index=3, name="Population"),
    )
)

c4 = (
    Grid(init_opts=opts.InitOpts(width="1200px", height="960px"))
    .add(
        chart=l1_1,
        grid_opts=opts.GridOpts(pos_right="57%", pos_bottom="57%"),
        grid_index=0,
    )
    .add(
        chart=l1_2,
        grid_opts=opts.GridOpts(pos_left="57%", pos_bottom="57%"),
        grid_index=1,
    )
    .add(
        chart=l2_1,
        grid_opts=opts.GridOpts(pos_right="57%", pos_top="57%"),
        grid_index=2,
    )
    .add(
        chart=l2_2, grid_opts=opts.GridOpts(pos_left="57%", pos_top="57%"), grid_index=3
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="dataset",
    public=True,
)

swanlab.log(
    {
        "Dataset - Dataset_pie": c1,
        "Dataset - Dataset_bar_0": c2,
        "Dataset - Dataset_bar_1": c3,
        "Dataset - Dataset_professional_scatter": c4
    }
)




"""
@author: cunyue
@file: runtime.py
@time: 2024/11/18 15:13
@description: 运行时信息，操作系统信息等
"""

import os
import platform
import socket
import subprocess
import sys

from swanlab.core_python import get_client
from swanlab.error import KeyFileError
from swanlab.package import get_key
from swanlab.swanlab_settings import get_settings


def get_runtime_info():
    if not get_settings().collect_runtime:
        return {}
    return {
        **get_computer_info(),
        **get_python_info(),
        **get_git_info(),
    }


def get_computer_info():
    return {
        "os": platform.platform(),
        "os_pretty_name": get_os_pretty_name(),
        "hostname": socket.gethostname(),
        "pid": os.getpid(),
        "cwd": os.getcwd(),
    }


def get_python_info():
    return {
        "python": platform.python_version(),
        "python_verbose": sys.version,
        "executable": sys.executable,
        "command": _get_command(),
    }


def _get_command() -> str:
    """获取当前执行的命令行字符串，并可选地屏蔽敏感API密钥

    Returns:
        str: 处理后的命令行字符串，敏感信息可能被屏蔽

    Note:
        - 当安全屏蔽设置启用且发现API密钥时，密钥会被替换为'****'
        - 自动尝试多种方式获取API密钥（HTTP接口 -> 密钥文件）
    """
    # 创建副本避免修改原始参数
    args = sys.argv.copy()

    # 仅在需要屏蔽时尝试获取API密钥
    if get_settings().security_mask:
        api_key = None

        # 尝试从不同来源获取API密钥
        # 未登录时 get_client 抛出 ValueError，此时需要换用 get_key 继续尝试获取
        # 未设置环境变量且未找到本地保存的 api key 时， get_key 抛出 KeyFileError，此时 api_key 为 None
        for key_provider in [lambda: get_client().api_key, get_key]:
            try:
                api_key = key_provider()
                break
            except (ValueError, KeyFileError):
                continue

        # 如果找到API密钥且在参数中，则进行屏蔽
        if api_key is not None and api_key in args:
            args[args.index(api_key)] = "****"

    return " ".join(args)


# ---------------------------------- git信息 ----------------------------------


def get_git_info():
    """获取git信息"""
    return {
        "git_remote": get_remote_url(),
        "git_info": get_git_branch_and_commit(),
    }


def get_os_pretty_name():
    """获取操作系统pretty name"""
    try:
        return platform.freedesktop_os_release().get("PRETTY_NAME", "")
    except Exception as e:  # noqa
        return None


def get_remote_url():
    """获取git仓库的远程链接

    Returns
    -------
    str
        git remote url
    """
    try:
        # 运行git命令获取远程仓库URL
        result = subprocess.run(
            ["git", "config", "--get", "remote.origin.url"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
        )

        # 检查命令是否成功运行
        if result.returncode == 0:
            url = result.stdout.strip()
            return parse_git_url(url)
        else:
            return None
    except Exception as e:  # noqa
        return None


def parse_git_url(url):
    """Return the remote URL of a git repository."""
    if url.startswith("git@"):
        parts = url[4:].split("/", 1)
        host, path = parts[0], parts[1] if len(parts) > 1 else ""
        if ":" in host:
            host, port = host.rsplit(":", 1)
            url = f"https://{host}:{port}/{path}" if port.isdigit() else f"https://{host}/{port}/{path}"
        else:
            url = f"https://{host}/{path}"
    return url[:-4] if url.endswith(".git") else url


def replace_second_colon(input_string, replacement):
    """Replace the second colon in a string."""
    first_colon = input_string.find(":")
    second_colon = input_string.find(":", first_colon + 1) if first_colon != -1 else -1
    return (
        input_string[:second_colon] + replacement + input_string[second_colon + 1 :]
        if second_colon != -1
        else input_string
    )


def get_git_branch_and_commit():
    """获取项目git的分支名和该分支下最新提交的hash"""
    try:
        # 获取当前分支名称
        branch_process = subprocess.Popen(
            ["git", "branch", "--show-current"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
        )
        branch_name, branch_err = branch_process.communicate()
        if branch_process.returncode != 0:
            return None, None

        branch_name = branch_name.strip()

        # 获取当前分支的最新提交hash
        commit_process = subprocess.Popen(
            ["git", "rev-parse", branch_name], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
        )
        commit_hash, commit_err = commit_process.communicate()

        # 如果无法获取最新提交hash, 那么就返回branch_name和None
        if commit_process.returncode != 0:
            return branch_name, None

        commit_hash = commit_hash.strip()
        return branch_name, commit_hash

    except Exception as e:  # noqa
        return None, None



"""
// @author: ComPleHN
// @file: scatter3d.vue
// @time: 2025/5/29 10:35
// @description: 本文件是对于echarts的 3D散点图 图表的测试
"""
# ---------------------------------------------- Scatter3d - Scatter3d ----------------------------------------------
import asyncio
from aiohttp import TCPConnector, ClientSession

import pyecharts.options as opts
from pyecharts.charts import Scatter3D

"""
Gallery 使用 pyecharts 1.1.0
参考地址: https://echarts.apache.org/examples/editor.html?c=scatter3d&gl=1&theme=dark

目前无法实现的功能:

1、暂时无法对 Grid3D 设置 轴线和轴坐标的 style (非白色背景下有问题)
"""


async def get_json_data(url: str) -> dict:
    async with ClientSession(connector=TCPConnector(ssl=False)) as session:
        async with session.get(url=url) as response:
            return await response.json()


# 获取官方的数据
data = asyncio.run(
    get_json_data(
        url="https://echarts.apache.org/examples/data/asset/data/nutrients.json"
    )
)

# 列名映射
field_indices = {
    "calcium": 3,
    "calories": 12,
    "carbohydrate": 8,
    "fat": 10,
    "fiber": 5,
    "group": 1,
    "id": 16,
    "monounsat": 14,
    "name": 0,
    "polyunsat": 15,
    "potassium": 7,
    "protein": 2,
    "saturated": 13,
    "sodium": 4,
    "sugars": 9,
    "vitaminc": 6,
    "water": 11,
}

# 配置 config
config_xAxis3D = "protein"
config_yAxis3D = "fiber"
config_zAxis3D = "sodium"
config_color = "fiber"
config_symbolSize = "vitaminc"

# 构造数据
data = [
    [
        item[field_indices[config_xAxis3D]],
        item[field_indices[config_yAxis3D]],
        item[field_indices[config_zAxis3D]],
        item[field_indices[config_color]],
        item[field_indices[config_symbolSize]],
        index,
    ]
    for index, item in enumerate(data)
]

c1 = (
    Scatter3D()  # bg_color="black"
    .add(
        series_name="",
        data=data,
        xaxis3d_opts=opts.Axis3DOpts(
            name=config_xAxis3D,
            type_="value",
            # textstyle_opts=opts.TextStyleOpts(color="#fff"),
        ),
        yaxis3d_opts=opts.Axis3DOpts(
            name=config_yAxis3D,
            type_="value",
            # textstyle_opts=opts.TextStyleOpts(color="#fff"),
        ),
        zaxis3d_opts=opts.Axis3DOpts(
            name=config_zAxis3D,
            type_="value",
            # textstyle_opts=opts.TextStyleOpts(color="#fff"),
        ),
        grid3d_opts=opts.Grid3DOpts(width=100, height=100, depth=100),
    )
    .set_global_opts(
        visualmap_opts=[
            opts.VisualMapOpts(
                type_="color",
                is_calculable=True,
                dimension=3,
                pos_top="10",
                max_=79 / 2,
                range_color=[
                    "#1710c0",
                    "#0b9df0",
                    "#00fea8",
                    "#00ff0d",
                    "#f5f811",
                    "#f09a09",
                    "#fe0300",
                ],
            ),
            opts.VisualMapOpts(
                type_="size",
                is_calculable=True,
                dimension=4,
                pos_bottom="10",
                max_=2.4 / 2,
                range_size=[10, 40],
            ),
        ]
    )
)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="scatter3d",
    public=True,
)

swanlab.log(
    {
        "Scatter3d - Scatter3d": c1
    }
)



"""
// @author: ComPleHN
// @file: overlap.vue
// @time: 2025/5/28 12:45
// @description: 本文件是对于echarts的 层叠组件 图表测试 
"""
# ---------------------------------------------- Overlap - Overlap_bar_line ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Line
from pyecharts.faker import Faker

v1 = [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3]
v2 = [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3]
v3 = [2.0, 2.2, 3.3, 4.5, 6.3, 10.2, 20.3, 23.4, 23.0, 16.5, 12.0, 6.2]


c1= (
    Bar()
    .add_xaxis(Faker.months)
    .add_yaxis("蒸发量", v1)
    .add_yaxis("降水量", v2)
    .extend_axis(
        yaxis=opts.AxisOpts(
            axislabel_opts=opts.LabelOpts(formatter="{value} °C"), interval=5
        )
    )
    .set_series_opts(label_opts=opts.LabelOpts(is_show=False))
    .set_global_opts(
        title_opts=opts.TitleOpts(title="Overlap-bar+line"),
        yaxis_opts=opts.AxisOpts(axislabel_opts=opts.LabelOpts(formatter="{value} ml")),
    )
)

line = Line().add_xaxis(Faker.months).add_yaxis("平均温度", v3, yaxis_index=1)
c1.overlap(line)

# ---------------------------------------------- Overlap - Overlap_line_scatter ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Line, Scatter
from pyecharts.faker import Faker

x = Faker.choose()
c2 = (
    Line()
    .add_xaxis(x)
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
    .set_global_opts(title_opts=opts.TitleOpts(title="Overlap-line+scatter"))
)
scatter = (
    Scatter()
    .add_xaxis(x)
    .add_yaxis("商家A", Faker.values())
    .add_yaxis("商家B", Faker.values())
)
c2.overlap(scatter)

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="overlap",
    public=True,
)

swanlab.log(
    {
        "Overlap - Overlap_bar_line": c1,
        "Overlap - Overlap_line_scatter": c2
    }
)



"""
// @author: ComPleHN
// @file: theme.vue
// @time: 2025/5/29 10:48
// @description: 本文件是对于echarts的 主题组件 图表的测试
"""
# ---------------------------------------------- Theme - Theme_example ----------------------------------------------
from pyecharts import options as opts
from pyecharts.charts import Bar, Page
from pyecharts.faker import Collector, Faker
from pyecharts.globals import ThemeType

C = Collector()


@C.funcs
def theme_default() -> Bar:
    c = (
        Bar()
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-default"))
    )
    return c


@C.funcs
def theme_light() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.LIGHT))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-light"))
    )
    return c


@C.funcs
def theme_dark() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.DARK))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-dark"))
    )
    return c


@C.funcs
def theme_chalk() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.CHALK))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-chalk"))
    )
    return c


@C.funcs
def theme_essos() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.ESSOS))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-essos"))
    )
    return c


@C.funcs
def theme_infographic() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.INFOGRAPHIC))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-infographic"))
    )
    return c


@C.funcs
def theme_macarons() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.MACARONS))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-macarons"))
    )
    return c


@C.funcs
def theme_purple_passion() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.PURPLE_PASSION))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-purple-passion"))
    )
    return c


@C.funcs
def theme_roma() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.ROMA))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-roma"))
    )
    return c


@C.funcs
def theme_romantic() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.ROMANTIC))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-romantic"))
    )
    return c


@C.funcs
def theme_shine() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.SHINE))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-shine"))
    )
    return c


@C.funcs
def theme_vintage() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.VINTAGE))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-vintage"))
    )
    return c


@C.funcs
def theme_walden() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.WALDEN))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-walden"))
    )
    return c


@C.funcs
def theme_westeros() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.WESTEROS))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-westeros"))
    )
    return c


@C.funcs
def theme_wonderland() -> Bar:
    c = (
        Bar(init_opts=opts.InitOpts(theme=ThemeType.WONDERLAND))
        .add_xaxis(Faker.choose())
        .add_yaxis("商家A", Faker.values())
        .add_yaxis("商家B", Faker.values())
        .add_yaxis("商家C", Faker.values())
        .add_yaxis("商家D", Faker.values())
        .set_global_opts(title_opts=opts.TitleOpts("Theme-wonderland"))
    )
    return c


c1 = Page().add(*[fn() for fn, _ in C.charts])

import swanlab

swanlab.init(
    project="echarts-test",
    experiment_name="theme",
    public=True,
)

swanlab.log(
    {
        "Theme - Theme_example": c1
    }
)



"""
@author: cunyue
@file: disabled.py
@time: 2025/6/24 14:21
@description: disabled 模式回调
"""

from swanlab.data.callbacker.callback import SwanLabRunCallback

from .. import namer as N


class DisabledCallback(SwanLabRunCallback):
    def on_init(self, proj_name: str, workspace: str, public: bool = None, logdir: str = None, *args, **kwargs):
        self.run_store.run_name = "run-disabled"
        self.run_store.run_colors = N.generate_colors(0)
        self.run_store.run_id = N.generate_run_id()
        self.run_store.new = True

    def __str__(self):
        return "SwanLabDisabledCallback"

    def on_stop(self, error: str = None, *args, **kwargs):
        self.porter.close()



"""
launch this scripts to use accelerate train a cifar classifier with swanlab tracker
launch command with default:
```
python train.py
```
launch with accelerate launcher:
```
accelerate launch train.py
```
or get more launch information from https://huggingface.co/docs/accelerate/basic_tutorials/launch

"""

import torch.utils
import torch.utils.data
import torch.utils.data.dataloader
from tutils import open_dev_mode
import swanlab
from swanlab.integration.accelerate import SwanLabTracker

swanlab.login(open_dev_mode())

import torch
from torchvision.models import resnet18, ResNet18_Weights
import torchvision

from accelerate import Accelerator
from accelerate.logging import get_logger


def main():
    # hyperparameters
    config = {
        "num_epoch": 5,
        "batch_num": 64,
        "learning_rate": 1e-3,
    }

    # Download the raw CIFAR-10 data.
    transform = torchvision.transforms.Compose(
        [
            torchvision.transforms.ToTensor(),
            torchvision.transforms.Normalize((0.485, 0.456, 0.406), (0.229, 0.224, 0.225)),
        ]
    )
    train_data = torchvision.datasets.CIFAR10(root="./data", train=True, download=True, transform=transform)
    test_data = torchvision.datasets.CIFAR10(root="./data", train=False, download=True, transform=transform)
    BATCH_SIZE = config["batch_num"]
    my_training_dataloader = torch.utils.data.DataLoader(train_data, batch_size=BATCH_SIZE, shuffle=True)
    my_testing_dataloader = torch.utils.data.DataLoader(test_data, batch_size=BATCH_SIZE, shuffle=False)

    # Using resnet18 model, make simple changes to fit the data set
    my_model = resnet18(weights=ResNet18_Weights.DEFAULT)
    my_model.conv1 = torch.nn.Conv2d(my_model.conv1.in_channels, my_model.conv1.out_channels, 3, 1, 1)
    my_model.maxpool = torch.nn.Identity()
    my_model.fc = torch.nn.Linear(my_model.fc.in_features, 10)

    # Criterion and optimizer
    criterion = torch.nn.CrossEntropyLoss()
    my_optimizer = torch.optim.SGD(my_model.parameters(), lr=config["learning_rate"], momentum=0.9)

    # Init accelerate with swanlab tracker
    tracker = SwanLabTracker("CIFAR10_TRAING")
    accelerator = Accelerator(log_with=tracker)
    accelerator.init_trackers("CIFAR10_TRAING", config=config)
    my_model, my_optimizer, my_training_dataloader, my_testing_dataloader = accelerator.prepare(
        my_model, my_optimizer, my_training_dataloader, my_testing_dataloader
    )
    device = accelerator.device
    my_model.to(device)

    # Get logger
    logger = get_logger(__name__)

    # Begin training

    for ep in range(config["num_epoch"]):
        # train model
        if accelerator.is_local_main_process:
            print(f"begin epoch {ep} training...")
        step = 0
        for stp, data in enumerate(my_training_dataloader):
            my_optimizer.zero_grad()
            inputs, targets = data
            outputs = my_model(inputs)
            loss = criterion(outputs, targets)
            accelerator.backward(loss)
            my_optimizer.step()
            if stp % 20 == 0:
                accelerator.log({"training_loss": loss, "epoch_num": ep}, step=ep * len(my_training_dataloader) + stp)
            if accelerator.is_local_main_process:
                print(f"train epoch {ep} [{stp}/{len(my_training_dataloader)}] | train loss {loss}")

        # eval model
        if accelerator.is_local_main_process:
            print(f"begin epoch {ep} evaluating...")
        total_acc_num = 0
        for stp, (inputs, targets) in enumerate(my_testing_dataloader):
            predictions = my_model(inputs)
            predictions = torch.argmax(predictions, dim=-1)
            # Gather all predictions and targets
            all_predictions, all_targets = accelerator.gather_for_metrics((predictions, targets))
            acc_num = (all_predictions.long() == all_targets.long()).sum()
            total_acc_num += acc_num
            if accelerator.is_local_main_process:
                print(f"eval epoch {ep} [{stp}/{len(my_testing_dataloader)}] | val acc {acc_num/len(all_targets)}")

        accelerator.log({"eval acc": total_acc_num / len(my_testing_dataloader.dataset)})

    accelerator.wait_for_everyone()
    accelerator.save_model(my_model, "cifar_cls.pth")

    accelerator.end_training()


if __name__ == "__main__":
    main()
